from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.checks.resource.base_spec_check import BaseK8Check


class ApiServerAdmissionControlEventRateLimit(BaseK8Check):
    def __init__(self) -> None:
        id = "CKV_K8S_78"
        name = "Ensure that the admission control plugin EventRateLimit is set"
        categories = (CheckCategories.KUBERNETES,)
        supported_kind = ('AdmissionConfiguration',)
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def scan_spec_conf(self, conf: dict[str, Any]) -> CheckResult:
        if "plugins" not in conf:
            return CheckResult.FAILED
        plugins = conf["plugins"]
        for plugin in plugins:
            if plugin["name"] == "EventRateLimit":
                return CheckResult.PASSED

        return CheckResult.FAILED


check = ApiServerAdmissionControlEventRateLimit()



from __future__ import annotations

from abc import abstractmethod
from collections.abc import Iterable
from typing import TYPE_CHECKING, Any

from checkov.ansible.checks.base_ansible_task_check import BaseAnsibleTaskCheck
from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckResult
from checkov.common.util.data_structures_utils import find_in_dict
from checkov.yaml_doc.enums import BlockType

if TYPE_CHECKING:
    from checkov.common.models.enums import CheckCategories


class BaseAnsibleTaskValueCheck(BaseAnsibleTaskCheck):
    def __init__(
        self,
        name: str,
        id: str,
        categories: Iterable[CheckCategories],
        supported_modules: Iterable[str],
        guideline: str | None = None,
        path: str | None = None,
        missing_block_result: CheckResult = CheckResult.FAILED,
    ) -> None:
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_modules=supported_modules,
            block_type=BlockType.ARRAY,
            guideline=guideline,
            path=path,
        )
        self.missing_block_result = missing_block_result

    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        inspected_key = self.get_inspected_key()
        expected_values = self.get_expected_values()

        value = find_in_dict(conf, inspected_key)

        if value is None:
            return self.missing_block_result, self.entity_conf
        if ANY_VALUE in expected_values:
            return CheckResult.PASSED, self.entity_conf
        if value in expected_values:
            return CheckResult.PASSED, self.entity_conf
        # quite often string values are case-insensitive
        if isinstance(value, str) and value.lower() in [exp.lower() for exp in expected_values if isinstance(exp, str)]:
            return CheckResult.PASSED, self.entity_conf

        return CheckResult.FAILED, self.entity_conf

    @abstractmethod
    def get_inspected_key(self) -> str:
        """
        :return: JSONPath syntax path of the checked attribute
        """
        raise NotImplementedError()

    def get_expected_values(self) -> list[Any]:
        """
        Override the method with the list of acceptable values if the check has more than one possible expected value, given
        the inspected key
        :return: List of expected values, defaults to a list of the expected value
        """
        return [self.get_expected_value()]

    def get_expected_value(self) -> Any:
        """
        Returns the default expected value, governed by provider best practices
        """
        return True

    def get_evaluated_keys(self) -> list[str]:
        return [self.get_inspected_key()]



from enum import Enum


class ResourceType(str, Enum):
    BLOCK = "block"
    TASKS = "tasks"

    def __str__(self) -> str:
        # needed, because of a Python 3.11 change
        return self.value



from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AppServicePublicAccessDisabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Azure Web App public network access is disabled"
        id = "CKV_AZURE_222"
        supported_resources = ("azurerm_linux_web_app", "azurerm_windows_web_app")
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "public_network_access_enabled"

    def get_expected_value(self) -> Any:
        return False


check = AppServicePublicAccessDisabled()



from __future__ import annotations

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class CDNEnableHttpsEndpoints(BaseResourceValueCheck):
    def __init__(self) -> None:
        # This is the full description of your check
        description = "Ensure the Azure CDN enables the HTTPS endpoint"

        # This is the Unique ID for your check
        id = "CKV_AZURE_198"

        # These are the Terraform objects supported by this check (ex: aws_iam_policy_document)
        supported_resources = ("azurerm_cdn_endpoint",)

        # Valid CheckCategories are defined in checkov/common/models/enums.py
        categories = (CheckCategories.NETWORKING,)
        super().__init__(
            name=description,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            missing_block_result=CheckResult.PASSED,
        )

    def get_inspected_key(self) -> str:
        return "is_https_allowed"


check = CDNEnableHttpsEndpoints()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class DLMScheduleCrossRegionEncryption(BaseResourceCheck):
    def __init__(self):
        name = "Ensure DLM cross region schedules are encrypted"
        id = "CKV_AWS_255"
        supported_resources = ['aws_dlm_lifecycle_policy']
        categories = [CheckCategories.BACKUP_AND_RECOVERY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if conf.get("policy_details") and isinstance(conf.get("policy_details"), list):
            policy = conf.get("policy_details")[0]

            if policy.get("schedule") and isinstance(policy.get("schedule"), list):
                schedules = policy.get("schedule")
                for idx, schedule in enumerate(schedules):
                    if schedule.get("cross_region_copy_rule") and isinstance(schedule.get("cross_region_copy_rule"), list):
                        for c_idx, cross_schedule_rule in enumerate(schedule.get("cross_region_copy_rule")):
                            if isinstance(cross_schedule_rule, dict) and cross_schedule_rule.get("encrypted") != [True]:
                                self.evaluated_keys = [
                                    f"policy_details/schedule/{idx}/cross_region_copy_rule/{c_idx}/encrypted"]
                                return CheckResult.FAILED
                        return CheckResult.PASSED
        return CheckResult.UNKNOWN


check = DLMScheduleCrossRegionEncryption()



from __future__ import annotations

from typing import Any, TYPE_CHECKING

from checkov.common.checks.base_check_registry import BaseCheckRegistry

if TYPE_CHECKING:
    from checkov.common.checks.base_check import BaseCheck
    from checkov.common.typing import _SkippedCheck, _CheckResult
    from checkov.runner_filter import RunnerFilter


class Registry(BaseCheckRegistry):
    def __init__(self, report_type: str) -> None:
        super().__init__(report_type)

    def extract_entity_details(self, entity: dict[str, Any]) -> tuple[str, dict[str, Any]]:  # type:ignore[override]
        kind = entity.get("kind") or ""
        conf = entity
        return kind, conf

    def scan(
        self,
        scanned_file: str,
        entity: dict[str, Any],
        skipped_checks: list[_SkippedCheck],
        runner_filter: RunnerFilter,
        report_type: str | None = None,
    ) -> dict[BaseCheck, _CheckResult]:
        (entity_type, entity_configuration) = self.extract_entity_details(entity)
        results = {}
        checks = self.get_checks(entity_type)
        for check in checks:
            skip_info: "_SkippedCheck" = {}
            if skipped_checks:
                if check.id in [x['id'] for x in skipped_checks]:
                    skip_info = [x for x in skipped_checks if x['id'] == check.id][0]

            if self._should_run_scan(check, entity_configuration, runner_filter, self.report_type):
                self.logger.debug("Running check: {} on file {}".format(check.name, scanned_file))

                result = check.run(scanned_file=scanned_file, entity_configuration=entity_configuration,
                                   entity_name=entity_type, entity_type=entity_type, skip_info=skip_info)
                results[check] = result
        return results

    @staticmethod
    def _should_run_scan(
        check: BaseCheck, entity_configuration: dict[str, Any], runner_filter: RunnerFilter, report_type: str
    ) -> bool:
        check_id_allowlist = runner_filter.checks
        check_id_denylist = runner_filter.skip_checks
        if check_id_allowlist or runner_filter.check_threshold:
            # Allow list provides namespace-only allows, check-only allows, or both
            # If namespaces not specified, all namespaces are scanned
            # If checks not specified, all checks are scanned

            if any("_" in check_id for check_id in check_id_allowlist) or runner_filter.check_threshold:
                # a Kubernetes namespace can't have an '_' in its name,
                # therefore we assume it is a built-in or custom check
                if not runner_filter.should_run_check(check=check, report_type=report_type):
                    return False

            allowed_namespaces = [check_id for check_id in check_id_allowlist if "_" not in check_id]
            if allowed_namespaces:
                # Check if namespace in allowed namespaces
                if "metadata" in entity_configuration and "namespace" in entity_configuration["metadata"]:
                    if entity_configuration["metadata"]["namespace"] in allowed_namespaces:
                        return True
                elif "parent_metadata" in entity_configuration and "namespace" in entity_configuration["parent_metadata"]:
                    if entity_configuration["parent_metadata"]["namespace"] in allowed_namespaces:
                        return True
                else:
                    if "default" in allowed_namespaces:
                        return True
            else:
                # No namespaces to filter
                return True
        elif check_id_denylist or runner_filter.skip_check_threshold or runner_filter.use_enforcement_rules:
            namespace_skip = False
            if "metadata" in entity_configuration and "namespace" in entity_configuration["metadata"]:
                if entity_configuration["metadata"]["namespace"] in check_id_denylist:
                    namespace_skip = True
            elif "parent_metadata" in entity_configuration and "namespace" in entity_configuration["parent_metadata"]:
                if entity_configuration["parent_metadata"]["namespace"] in check_id_denylist:
                    namespace_skip = True
            else:
                if "default" in check_id_denylist:
                    namespace_skip = True
            if runner_filter.should_run_check(check=check, report_type=report_type) and not namespace_skip:
                return True
        else:
            return True
        return False



from enum import Enum


class ResourceType(str, Enum):
    ADD = "ADD"
    ARG = "ARG"
    CMD = "CMD"
    COPY = "COPY"
    ENTRYPOINT = "ENTRYPOINT"
    ENV = "ENV"
    EXPOSE = "EXPOSE"
    FROM = "FROM"
    HEALTHCHECK = "HEALTHCHECK"
    LABEL = "LABEL"
    MAINTAINER = "MAINTAINER"
    ONBUILD = "ONBUILD"
    RUN = "RUN"
    SHELL = "SHELL"
    STOPSIGNAL = "STOPSIGNAL"
    USER = "USER"
    VOLUME = "VOLUME"
    WORKDIR = "WORKDIR"

    def __str__(self) -> str:
        # needed, because of a Python 3.11 change
        return self.value



from __future__ import annotations

from typing import Any

from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class DMSEndpointUsesCMK(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure DMS endpoint uses Customer Managed Key (CMK)"
        id = "CKV_AWS_296"
        supported_resources = ("aws_dms_endpoint",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        engine_name = conf.get("engine_name")
        if engine_name and isinstance(engine_name, list) and engine_name[0] == "s3":
            self.evaluated_keys = ["s3_settings"]
            s3_settings = conf.get("s3_settings")
            if s3_settings and isinstance(s3_settings, list):
                self.evaluated_keys = ["s3_settings/server_side_encryption_kms_key_id"]
                settings = s3_settings[0]
                if settings.get("server_side_encryption_kms_key_id"):
                    return CheckResult.PASSED
            return CheckResult.FAILED

        self.evaluated_keys = ["kms_key_arn"]
        kms_key = conf.get("kms_key_arn")
        if kms_key and isinstance(kms_key, list) and kms_key[0]:
            return CheckResult.PASSED

        return CheckResult.FAILED


check = DMSEndpointUsesCMK()



from __future__ import annotations

from dataclasses import dataclass
from typing import Literal

from checkov.common.graph.graph_builder.graph_components.block_types import BlockType as CommonBlockType


@dataclass
class BlockType(CommonBlockType):
    PARAMETER: Literal["parameter"] = "parameter"



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class SecurityGroupRuleDescription(BaseResourceCheck):
    def __init__(self):
        name = "Ensure every security groups rule has a description"
        id = "CKV_AWS_23"
        supported_resource = [
            'aws_security_group',
            'aws_security_group_rule',
            'aws_db_security_group',
            'aws_elasticache_security_group',
            'aws_redshift_security_group',
            'aws_vpc_security_group_egress_rule',
            'aws_vpc_security_group_ingress_rule'
        ]
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resource)

    def scan_resource_conf(self, conf):
        """
            Looks for description at security group  rules :
            https://www.terraform.io/docs/providers/aws/r/security_group.html
        :param conf: aws_security_group configuration
        :return: <CheckResult>
        """
        group_result = self.check_rule(rule_type='group_or_rule_description', conf=conf)
        if 'type' not in conf.keys():
            # 'type' attribute only exists and is required for aws_security_group_rule resources.
            # Therefore, only if 'type' is not present, ingress and egress blocks must be checked in the resource.
            egress_result = self.check_rule(rule_type='egress', conf=conf)
            ingress_result = self.check_rule(rule_type='ingress', conf=conf)
            if group_result == CheckResult.PASSED and egress_result == CheckResult.PASSED and ingress_result == CheckResult.PASSED:
                return CheckResult.PASSED
            return CheckResult.FAILED

        return group_result

    def check_rule(self, rule_type, conf):
        if rule_type == 'group_or_rule_description':
            if 'description' in conf.keys():
                self.evaluated_keys = ['description']
                if conf['description']:
                    return CheckResult.PASSED
            return CheckResult.FAILED

        if rule_type in conf.keys():
            for rule in conf[rule_type]:
                if isinstance(rule, dict) and ('description' not in rule.keys() or not rule['description']):
                    self.evaluated_keys.append(f'{rule_type}/[{conf[rule_type].index(rule)}]')
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = SecurityGroupRuleDescription()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AppServiceSlotDebugDisabled(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure debugging is disabled for the App service slot"
        id = "CKV_AZURE_155"
        supported_resources = ['azurerm_app_service_slot']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self):
        return "site_config/[0]/remote_debugging_enabled/[0]"

    def get_expected_value(self):
        return False


check = AppServiceSlotDebugDisabled()



from typing import Dict, List, Any

from checkov.common.util.type_forcers import force_int

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck

ASYMMETRIC_KEYS = {"ASYMMETRIC_DECRYPT", "ASYMMETRIC_SIGN"}
# rotation_period time unit is seconds
ONE_DAY = 24 * 60 * 60
NINETY_DAYS = 90 * ONE_DAY


class GoogleKMSKeyRotationPeriod(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure KMS encryption keys are rotated within a period of 90 days"
        id = "CKV_GCP_43"
        supported_resources = ("google_kms_crypto_key",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        purpose = conf.get("purpose")
        if purpose and isinstance(purpose, list) and purpose[0] in ASYMMETRIC_KEYS:
            # https://cloud.google.com/kms/docs/key-rotation#asymmetric
            # automatic key rotation is not supported for asymmetric keys
            return CheckResult.UNKNOWN

        self.evaluated_keys = ["rotation_period"]
        rotation = conf.get("rotation_period")
        if rotation and rotation[0] and isinstance(rotation[0], str):
            time = force_int(rotation[0][:-1])
            if time and ONE_DAY <= time <= NINETY_DAYS:
                return CheckResult.PASSED
        return CheckResult.FAILED


check = GoogleKMSKeyRotationPeriod()



from __future__ import annotations

import logging
from typing import Any, TYPE_CHECKING

from checkov.arm.graph_builder.graph_components.block_types import BlockType
from checkov.arm.graph_builder.graph_components.blocks import ArmBlock
from checkov.arm.utils import ArmElements
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.graph.graph_builder.local_graph import LocalGraph
from checkov.common.util.consts import START_LINE, END_LINE
from checkov.common.util.data_structures_utils import pickle_deepcopy

if TYPE_CHECKING:
    from checkov.common.graph.graph_builder.local_graph import _Block


class ArmLocalGraph(LocalGraph[ArmBlock]):
    def __init__(self, definitions: dict[str, dict[str, Any]]) -> None:
        super().__init__()
        self.vertices: list[ArmBlock] = []
        self.definitions = definitions
        self.vertices_by_path_and_id: dict[tuple[str, str], int] = {}

    def build_graph(self, render_variables: bool = False) -> None:
        self._create_vertices()
        logging.debug(f"[ArmLocalGraph] created {len(self.vertices)} vertices")

        self._create_edges()
        logging.debug(f"[ArmLocalGraph] created {len(self.edges)} edges")

    def _create_vertices(self) -> None:
        for file_path, definition in self.definitions.items():
            self._create_parameter_vertices(file_path=file_path, parameters=definition.get(ArmElements.PARAMETERS))
            self._create_resource_vertices(file_path=file_path, resources=definition.get(ArmElements.RESOURCES))

        for i, vertex in enumerate(self.vertices):
            self.vertices_by_block_type[vertex.block_type].append(i)
            self.vertices_block_name_map[vertex.block_type][vertex.name].append(i)
            self.vertices_by_path_and_id[(vertex.path, vertex.id)] = i

    def _create_parameter_vertices(self, file_path: str, parameters: dict[str, dict[str, Any]] | None) -> None:
        if not parameters:
            return

        for name, config in parameters.items():
            if name in (START_LINE, END_LINE):
                continue
            if not isinstance(config, dict):
                logging.debug(f"[ArmLocalGraph] parameter {name} has wrong type {type(config)}")
                continue

            attributes = pickle_deepcopy(config)

            self.vertices.append(
                ArmBlock(
                    name=name,
                    config=config,
                    path=file_path,
                    block_type=BlockType.PARAMETER,
                    attributes=attributes,
                    id=f"{BlockType.PARAMETER}.{name}",
                )
            )

    def _create_resource_vertices(self, file_path: str, resources: list[dict[str, Any]] | None) -> None:
        if not resources:
            return

        for config in resources:
            if "type" not in config:
                # this can't be a real ARM resource without a "type" field
                return

            resource_name = config.get("name") or "unknown"
            resource_type = config["type"]

            attributes = pickle_deepcopy(config)
            attributes[CustomAttributes.RESOURCE_TYPE] = resource_type

            self.vertices.append(
                ArmBlock(
                    name=resource_name,
                    config=config,
                    path=file_path,
                    block_type=BlockType.RESOURCE,
                    attributes=attributes,
                    id=f"{resource_type}.{resource_name}",
                )
            )

    def _create_edges(self) -> None:
        # no edges yet
        pass

    def update_vertices_configs(self) -> None:
        # not used
        pass

    @staticmethod
    def update_vertex_config(
        vertex: _Block, changed_attributes: list[str] | dict[str, Any], has_dynamic_blocks: bool = False
    ) -> None:
        # not used
        pass

    def get_resources_types_in_graph(self) -> list[str]:
        # not used
        return []



from enum import Enum
from typing import Dict, List, Union

from checkov.common.bridgecrew.severities import Severity, BcSeverities, Severities
from checkov.common.bridgecrew.check_type import CheckType


class CodeCategoryType(str, Enum):
    IAC = "IAC"
    VULNERABILITIES = "VULNERABILITIES"
    SECRETS = "SECRETS"
    LICENSES = "LICENSES"
    BUILD_INTEGRITY = "BUILD_INTEGRITY"


CodeCategoryMapping: Dict[str, Union[CodeCategoryType, List[CodeCategoryType]]] = {
    CheckType.ANSIBLE: CodeCategoryType.IAC,
    CheckType.ARGO_WORKFLOWS: CodeCategoryType.BUILD_INTEGRITY,
    CheckType.ARM: CodeCategoryType.IAC,
    CheckType.AZURE_PIPELINES: CodeCategoryType.BUILD_INTEGRITY,
    CheckType.BICEP: CodeCategoryType.IAC,
    CheckType.BITBUCKET_PIPELINES: CodeCategoryType.BUILD_INTEGRITY,
    CheckType.CIRCLECI_PIPELINES: CodeCategoryType.BUILD_INTEGRITY,
    CheckType.CLOUDFORMATION: CodeCategoryType.IAC,
    CheckType.DOCKERFILE: CodeCategoryType.IAC,
    CheckType.GITHUB_CONFIGURATION: CodeCategoryType.BUILD_INTEGRITY,
    CheckType.GITHUB_ACTIONS: CodeCategoryType.BUILD_INTEGRITY,
    CheckType.GITLAB_CONFIGURATION: CodeCategoryType.BUILD_INTEGRITY,
    CheckType.GITLAB_CI: CodeCategoryType.BUILD_INTEGRITY,
    CheckType.BITBUCKET_CONFIGURATION: CodeCategoryType.BUILD_INTEGRITY,
    CheckType.HELM: CodeCategoryType.IAC,
    CheckType.JSON: CodeCategoryType.IAC,
    CheckType.YAML: CodeCategoryType.IAC,
    CheckType.KUBERNETES: CodeCategoryType.IAC,
    CheckType.KUSTOMIZE: CodeCategoryType.IAC,
    CheckType.OPENAPI: CodeCategoryType.IAC,
    CheckType.SCA_PACKAGE: [CodeCategoryType.LICENSES, CodeCategoryType.VULNERABILITIES],
    CheckType.SCA_IMAGE: [CodeCategoryType.LICENSES, CodeCategoryType.VULNERABILITIES],
    CheckType.SECRETS: CodeCategoryType.SECRETS,
    CheckType.SERVERLESS: CodeCategoryType.IAC,
    CheckType.TERRAFORM: CodeCategoryType.IAC,
    CheckType.TERRAFORM_JSON: CodeCategoryType.IAC,
    CheckType.TERRAFORM_PLAN: CodeCategoryType.IAC,
    CheckType.POLICY_3D: CodeCategoryType.IAC
}


class CodeCategoryConfiguration:
    def __init__(self, category: str, soft_fail_threshold: Severity, hard_fail_threshold: Severity):
        self.category = category
        self.soft_fail_threshold = soft_fail_threshold
        self.hard_fail_threshold = hard_fail_threshold

    def is_global_soft_fail(self) -> bool:
        return self.hard_fail_threshold == Severities[BcSeverities.OFF]



from __future__ import annotations

import os

from pathlib import Path

home = str(Path.home())
bridgecrew_dir = f"{home}/.bridgecrew"
bridgecrew_file = f"{bridgecrew_dir}/credentials"


def persist_key(key: str) -> None:
    if not os.path.exists(bridgecrew_dir):
        os.makedirs(bridgecrew_dir)
    with open(bridgecrew_file, "w") as f:
        f.write(key)


def read_key() -> str | None:
    key = None
    if os.path.exists(bridgecrew_file):
        with open(bridgecrew_file, "r") as f:
            key = f.readline() or None  # in Mac, if the credentials file is empty, f.readline() == '' and it causes bugs
    return key



from __future__ import annotations

from typing import Any
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.checks.enums import BlockType
from checkov.openapi.checks.resource.v2.BaseOpenapiCheckV2 import BaseOpenapiCheckV2


class GlobalSecurityScopeUndefined(BaseOpenapiCheckV2):
    def __init__(self) -> None:
        id = "CKV_OPENAPI_19"
        name = "Ensure that global security scope is defined in securityDefinitions - version 2.0 files"
        categories = [CheckCategories.API_SECURITY]
        supported_resources = ["security"]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_resources,
            block_type=BlockType.DOCUMENT,
        )

    def scan_openapi_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]]:
        security_definitions = conf.get('securityDefinitions') or {}
        security_values = conf.get('security') or [{}]
        for security in security_values:
            if not isinstance(security, dict):
                return CheckResult.UNKNOWN, conf
            for security_key, security_scopes in security.items():
                if self.is_start_end_line(security_key) or not security_scopes:
                    continue
                security_definition = security_definitions.get(security_key, {})
                if not security_definition:
                    return CheckResult.FAILED, conf
                definition_scopes = security_definition.get('scopes', {})
                if not definition_scopes:
                    return CheckResult.FAILED, conf
                for scope in security_scopes:
                    if scope not in definition_scopes:
                        return CheckResult.FAILED, conf

        return CheckResult.PASSED, conf


check = GlobalSecurityScopeUndefined()



from __future__ import annotations

from typing import Any

from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class SQLServerUsesADAuth(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        """
        I think that this check is really, ensure that only AD auth is used (not user/pass)
        """

        name = "Ensure Azure AD authentication is enabled for Azure SQL (MSSQL)"
        id = "CKV2_AZURE_27"
        supported_resources = ["Microsoft.Sql/servers"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'properties/administratorLogin'

    def get_forbidden_values(self) -> list[Any]:
        return [ANY_VALUE]


check = SQLServerUsesADAuth()



from __future__ import annotations

from typing import Any

from checkov.arm.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from checkov.common.models.enums import CheckCategories


class StorageAccountDisablePublicAccess(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Storage accounts disallow public access"
        id = "CKV_AZURE_59"
        supported_resources = ("Microsoft.Storage/storageAccounts",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "properties/publicNetworkAccess"

    def get_forbidden_values(self) -> list[Any]:
        return ["Enabled"]


check = StorageAccountDisablePublicAccess()



from __future__ import annotations

from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class AppServiceEnableFailedRequest(BaseResourceValueCheck):
    def __init__(self) -> None:
        """
        todo: revisit when graph fully enabled as web config section could be missing entirely from a web app
        """

        name = "Ensure that App service enables failed request tracing"
        id = "CKV_AZURE_66"
        supported_resources = ["Microsoft.Web/sites/config"]
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "properties/requestTracingEnabled"


check = AppServiceEnableFailedRequest()



from __future__ import annotations

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck


class TransferServerAllowsOnlySecureProtocols(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Transfer Server allows only secure protocols"
        id = "CKV_AWS_357"
        supported_resources = ("aws_transfer_server",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        protocols = conf.get("protocols")
        if protocols and isinstance(protocols, list):
            if "FTP" in protocols[0]:
                return CheckResult.FAILED
        return CheckResult.PASSED


check = TransferServerAllowsOnlySecureProtocols()



from __future__ import annotations


from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class SynapseWorkspaceEnablesManagedVirtualNetworks(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Azure Synapse workspaces enables managed virtual networks"
        id = "CKV_AZURE_58"
        supported_resources = ['Microsoft.Synapse/workspaces']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'properties/managedVirtualNetwork'

    def get_forbidden_values(self) -> str:
        return "default"


check = SynapseWorkspaceEnablesManagedVirtualNetworks()



from __future__ import annotations

from typing import Any

from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class NeptuneClusterBackupRetention(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that Neptune DB cluster has automated backups enabled with adequate retention"
        id = "CKV_AWS_361"
        supported_resources = ("AWS::Neptune::DBCluster",)
        categories = (CheckCategories.BACKUP_AND_RECOVERY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        backup_retention = conf.get("Properties", {}).get("BackupRetentionPeriod", 1)
        if backup_retention >= 7:
            return CheckResult.PASSED

        return CheckResult.FAILED

    def get_evaluated_keys(self) -> list[str]:
        return ["Properties/BackupRetentionPeriod"]


check = NeptuneClusterBackupRetention()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class CodeBuildProjectEncryption(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that CodeBuild Project encryption is not disabled"
        id = "CKV_AWS_78"
        supported_resources = ("AWS::CodeBuild::Project",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        # Only Fail if Artifact Type is S3 and EncryptionDisabled is True.
        artifact_type = ""
        encryption_disabled = False
        properties = conf.get("Properties")
        if properties:
            artifacts = properties.get("Artifacts")
            if artifacts and isinstance(artifacts, dict):
                if "Type" in artifacts.keys():
                    artifact_type = artifacts["Type"]
                if "EncryptionDisabled" in artifacts.keys():
                    encryption_disabled = artifacts["EncryptionDisabled"]
                if artifact_type == "S3" and encryption_disabled is True:
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = CodeBuildProjectEncryption()



from __future__ import annotations

import dataclasses
import logging
import os
from typing import Any, TYPE_CHECKING, Optional

from typing_extensions import TypeAlias  # noqa[TC002]

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.graph.checks_infra.registry import BaseRegistry
from checkov.common.graph.graph_builder.consts import GraphSource
from checkov.common.output.extra_resource import ExtraResource
from checkov.common.parallelizer.parallel_runner import parallel_runner
from checkov.common.output.graph_record import GraphRecord
from checkov.common.output.record import Record
from checkov.common.output.report import Report, merge_reports, remove_duplicate_results
from checkov.common.util import data_structures_utils
from checkov.common.util.consts import RESOLVED_MODULE_ENTRY_NAME
from checkov.terraform import get_module_from_full_path, get_module_name, get_abs_path
from checkov.common.util.secrets import omit_secret_value_from_checks
from checkov.runner_filter import RunnerFilter
from checkov.terraform.base_runner import BaseTerraformRunner
from checkov.terraform.graph_manager import TerraformGraphManager
from checkov.terraform.modules.module_objects import TFDefinitionKey, TFModule
from checkov.terraform.context_parsers.registry import parser_registry
from checkov.terraform.evaluation.base_variable_evaluation import BaseVariableEvaluation
from checkov.common.graph.graph_builder.graph_components.attribute_names import CustomAttributes
from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.graph_to_tf_definitions import convert_graph_vertices_to_tf_definitions
from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph
from checkov.terraform.tag_providers import get_resource_tags
from checkov.common.runners.base_runner import strtobool
from checkov.terraform.tf_parser import TFParser

if TYPE_CHECKING:
    from checkov.common.typing import _SkippedCheck, LibraryGraph, LibraryGraphConnector

_TerraformContext: TypeAlias = "dict[TFDefinitionKey, dict[str, Any]]"
_TerraformDefinitions: TypeAlias = "dict[TFDefinitionKey, dict[str, Any]]"

CHECK_BLOCK_TYPES = frozenset(["resource", "data", "provider", "module"])


class Runner(BaseTerraformRunner[_TerraformDefinitions, _TerraformContext, TFDefinitionKey]):
    check_type = CheckType.TERRAFORM  # noqa: CCE003  # a static attribute

    def __init__(
        self,
        parser: TFParser | None = None,
        db_connector: LibraryGraphConnector | None = None,
        external_registries: list[BaseRegistry] | None = None,
        source: str = GraphSource.TERRAFORM,
        graph_class: type[TerraformLocalGraph] = TerraformLocalGraph,
        graph_manager: TerraformGraphManager | None = None,
    ) -> None:
        super().__init__(parser, db_connector, external_registries, source, graph_class, graph_manager)
        self.all_graphs: list[tuple[LibraryGraph, Optional[str]]] = []

    def run(
        self,
        root_folder: str | None,
        external_checks_dir: list[str] | None = None,
        files: list[str] | None = None,
        runner_filter: RunnerFilter | None = None,
        collect_skip_comments: bool = True,
    ) -> Report | list[Report]:
        runner_filter = runner_filter or RunnerFilter()
        if not runner_filter.show_progress_bar:
            self.pbar.turn_off_progress_bar()

        report = Report(self.check_type)
        parsing_errors: dict[str, Exception] = {}
        self.load_external_checks(external_checks_dir)
        if self.context is None or self.definitions is None or self.breadcrumbs is None:
            self.definitions = {}
            logging.info("Scanning root folder and producing fresh tf_definitions and context")
            tf_split_graph = strtobool(os.getenv("TF_SPLIT_GRAPH", "False"))
            if root_folder:
                root_folder = os.path.abspath(root_folder)
                if tf_split_graph:
                    graphs_with_definitions, self.resource_subgraph_map = self.graph_manager.build_multi_graph_from_source_directory(
                        source_dir=root_folder,
                        local_graph_class=self.graph_class,
                        download_external_modules=runner_filter.download_external_modules,
                        external_modules_download_path=runner_filter.external_modules_download_path,
                        parsing_errors=parsing_errors,
                        excluded_paths=runner_filter.excluded_paths,
                        vars_files=runner_filter.var_files,
                    )
                    local_graphs: list[tuple[str | None, TerraformLocalGraph]] = []
                    for graph, definitions, subgraph_path in graphs_with_definitions:
                        for definition in definitions:
                            self.definitions.update(definition)
                        local_graphs.append((subgraph_path, graph))
                else:
                    single_graph, self.definitions = self.graph_manager.build_graph_from_source_directory(
                        source_dir=root_folder,
                        local_graph_class=self.graph_class,
                        download_external_modules=runner_filter.download_external_modules,
                        external_modules_download_path=runner_filter.external_modules_download_path,
                        parsing_errors=parsing_errors,
                        excluded_paths=runner_filter.excluded_paths,
                        vars_files=runner_filter.var_files,
                    )
                    # Make graph a list to allow single processing method for all cases
                    local_graphs = [(None, single_graph)]
            elif files:
                files = [os.path.abspath(file) for file in files]
                root_folder = os.path.split(os.path.commonprefix(files))[0]
                self._parse_files(files, parsing_errors)

                if tf_split_graph:
                    local_graphs = self.graph_manager.build_multi_graph_from_definitions(self.definitions)
                else:
                    # local_graph needs to be a list to allow supporting multi graph
                    local_graphs = [(None, self.graph_manager.build_graph_from_definitions(self.definitions))]
            else:
                raise Exception("Root directory was not specified, files were not specified")

            if local_graphs:
                self._update_definitions_and_breadcrumbs(
                    local_graphs,
                    report,
                    root_folder)
        else:
            logging.info("Scanning root folder using existing tf_definitions")
            if root_folder is None:
                # this shouldn't happen
                raise Exception("Root directory was not specified")

        self.pbar.initiate(len(self.definitions))
        self.check_tf_definition(report, root_folder, runner_filter, collect_skip_comments)

        report.add_parsing_errors(parsing_errors.keys())

        if self.all_graphs:
            for igraph_graph, _ in self.all_graphs:
                graph_report = self.get_graph_checks_report(root_folder, runner_filter, graph=igraph_graph)
                merge_reports(report, graph_report)
        else:
            graph_report = self.get_graph_checks_report(root_folder, runner_filter)
            merge_reports(report, graph_report)

        report = remove_duplicate_results(report)

        if runner_filter.run_image_referencer:
            image_report = self.check_container_image_references(
                graph_connector=self.graph_manager.get_reader_endpoint(),
                root_path=root_folder,
                runner_filter=runner_filter,
            )

            if image_report:
                # due too many tests failing only return a list, if there is an image report
                return [report, image_report]

        return report

    def _parse_files(self, files: list[str], parsing_errors: dict[str, Exception]) -> None:
        if self.definitions is None:
            # just make sure it is not 'None'
            self.definitions = {}

        def parse_file(file: str) -> tuple[str, dict[str, Any] | None, dict[str, Exception]] | None:
            if not (file.endswith(".tf") or file.endswith(".hcl")):
                return None
            file_parsing_errors: dict[str, Exception] = {}
            parse_result = self.parser.parse_file(file=file, parsing_errors=file_parsing_errors)
            # the exceptions type can un-pickleable so we need to cast them to Exception
            for path, e in file_parsing_errors.items():
                file_parsing_errors[path] = Exception(e.__repr__())
            return file, parse_result, file_parsing_errors

        results = parallel_runner.run_function(parse_file, files)
        for result in results:
            if result:
                file, parse_result, file_parsing_errors = result
                if parse_result is not None:
                    self.definitions[TFDefinitionKey(file_path=file)] = parse_result
                if file_parsing_errors:
                    parsing_errors.update(file_parsing_errors)

    def _update_definitions_and_breadcrumbs(
        self, local_graphs: list[tuple[Optional[str], TerraformLocalGraph]], report: Report, root_folder: str
    ) -> None:
        self.definitions = {}
        self.breadcrumbs = {}
        self.all_graphs = []
        for subgraph_path, graph in local_graphs:
            for vertex in graph.vertices:
                if vertex.block_type == BlockType.RESOURCE:
                    vertex_id = vertex.attributes.get(CustomAttributes.TF_RESOURCE_ADDRESS)
                    report.add_resource(f"{vertex.path}:{vertex_id}")
            igraph_graph = self.graph_manager.save_graph(graph)
            self.all_graphs.append((igraph_graph, subgraph_path))
            current_definitions, current_breadcrumbs = convert_graph_vertices_to_tf_definitions(
                graph.vertices,
                root_folder,
            )
            self.definitions.update(current_definitions)
            self.breadcrumbs.update(current_breadcrumbs)

    def check_tf_definition(
        self,
        report: Report,
        root_folder: str,
        runner_filter: RunnerFilter,
        collect_skip_comments: bool = True,
    ) -> None:
        parser_registry.reset_definitions_context()
        if not self.definitions:
            # nothing to do
            self.pbar.update()
            self.pbar.close()
            return

        if not self.context:
            definitions_context = {}
            for definition_key_tuple in self.definitions.items():
                definitions_context = parser_registry.enrich_definitions_context(
                    definitions=definition_key_tuple, collect_skip_comments=collect_skip_comments
                )
            self.context = definitions_context
            logging.debug("Created definitions context")

        self.push_skipped_checks_down_from_modules(self.context)
        for full_file_path, definition in self.definitions.items():
            self.pbar.set_additional_data({"Current File Scanned": os.path.relpath(full_file_path.file_path)})
            abs_scanned_file = get_abs_path(full_file_path)
            abs_referrer = None
            scanned_file = f"{os.sep}{os.path.relpath(abs_scanned_file, root_folder)}"
            logging.debug(f"Scanning file: {scanned_file}")
            self.run_all_blocks(
                definition, self.context, full_file_path, root_folder, report, scanned_file, runner_filter, abs_referrer
            )
            self.pbar.update()
        self.pbar.close()

    def push_skipped_checks_down_from_modules(self, definition_context: dict[TFDefinitionKey, dict[str, Any]]) -> None:
        if not self.definitions:
            # no need to proceed
            return

        module_context_parser = parser_registry.context_parsers[BlockType.MODULE]
        for tf_definition_key, definition in self.definitions.items():
            full_file_path = tf_definition_key
            definition_modules_context = definition_context.get(full_file_path, {}).get(BlockType.MODULE, {})
            for entity in definition.get(BlockType.MODULE, []):
                module_name = module_context_parser.get_entity_context_path(entity)[0]
                skipped_checks = definition_modules_context.get(module_name, {}).get("skipped_checks")
                resolved_paths = entity.get(module_name).get(RESOLVED_MODULE_ENTRY_NAME)
                self.push_skipped_checks_down(definition_context, skipped_checks, resolved_paths)

    def push_skipped_checks_down(
        self,
        definition_context: dict[TFDefinitionKey, dict[str, Any]],
        skipped_checks: list[_SkippedCheck],
        resolved_paths: list[TFDefinitionKey],
    ) -> None:
        # this method pushes the skipped_checks down the 1 level to all resource types.
        if not skipped_checks or not resolved_paths:
            return
        for ind, definition in enumerate(resolved_paths):
            for block_type, block_configs in definition_context.get(definition, {}).items():
                # skip if type is not a Terraform resource
                if block_type not in CHECK_BLOCK_TYPES:
                    continue

                if block_type == "module":
                    if not self.definitions:
                        # no need to proceed
                        continue

                    # modules don't have a type, just a name
                    for module_name, module_config in block_configs.items():
                        # append the skipped checks also from a module to another module
                        module_config["skipped_checks"] += skipped_checks
                        module_context = next(
                            m
                            for m in self.definitions.get(resolved_paths[ind], {}).get(block_type, [])
                            if module_name in m
                        )
                        recursive_resolved_paths = module_context.get(module_name).get(RESOLVED_MODULE_ENTRY_NAME)
                        self.push_skipped_checks_down(definition_context, skipped_checks, recursive_resolved_paths)
                else:
                    # there may be multiple resource types - aws_bucket, etc
                    for resource_configs in block_configs.values():
                        # there may be multiple names for each resource type
                        for resource_config in resource_configs.values():
                            # append the skipped checks from the module to the other resources.
                            resource_config["skipped_checks"] += skipped_checks

    def run_all_blocks(
        self,
        definition: dict[str, list[dict[str, Any]]],
        definitions_context: _TerraformContext,
        full_file_path: TFDefinitionKey,
        root_folder: str,
        report: Report,
        scanned_file: str,
        runner_filter: RunnerFilter,
        module_referrer: str | None,
    ) -> None:
        if not definition:
            logging.debug(f"Empty definition, skipping run (root_folder={root_folder})")
            return
        block_types = set(definition.keys())
        for block_type in block_types & CHECK_BLOCK_TYPES:
            self.run_block(
                definition[block_type],
                definitions_context,
                full_file_path,
                root_folder,
                report,
                scanned_file,
                block_type,
                runner_filter,
                None,
                module_referrer,
            )

    def run_block(
        self,
        entities: list[dict[str, Any]],
        definition_context: _TerraformContext,
        full_file_path: TFDefinitionKey,
        root_folder: str,
        report: Report,
        scanned_file: str,
        block_type: str,
        runner_filter: RunnerFilter,
        entity_context_path_header: str | None = None,
        module_referrer: str | None = None,
    ) -> None:
        registry = self.block_type_registries[block_type]
        if not registry:
            return

        for entity in entities:
            entity_evaluations = None
            context_parser = parser_registry.context_parsers[block_type]
            definition_path = context_parser.get_entity_context_path(entity)
            (entity_type, entity_name, entity_config) = registry.extract_entity_details(entity)

            caller_file_path = None
            caller_file_line_range = None

            entity_id = entity_config.get(CustomAttributes.TF_RESOURCE_ADDRESS)
            module_full_path, _ = get_module_from_full_path(full_file_path)
            if module_full_path:
                module_name = get_module_name(full_file_path)
                if not module_name:
                    full_definition_path = entity_id.split(".")
                    try:
                        module_name_index = (
                            len(full_definition_path) - full_definition_path[::-1][1:].index(BlockType.MODULE) - 1
                        )  # the next item after the last 'module' prefix is the module name
                    except ValueError as e:
                        # TODO handle multiple modules with the same name in repo
                        logging.warning(f"Failed to get module name for resource {entity_id}. {str(e)}")
                        continue
                    module_name = full_definition_path[module_name_index]
                caller_context = definition_context[module_full_path].get(BlockType.MODULE, {}).get(module_name)
                if not caller_context:
                    continue
                caller_file_line_range = (caller_context.get("start_line", 1), caller_context.get("end_line", 1))
                abs_caller_file = get_abs_path(module_full_path)
                caller_file_path = f"{os.sep}{os.path.relpath(abs_caller_file, root_folder)}"

            if entity_context_path_header is None:
                entity_context_path = [block_type] + definition_path
            else:
                # TODO: check, if this code part is still used
                entity_context_path = [entity_context_path_header, block_type] + definition_path
            # Entity can exist only once per dir, for file as well
            context_path = full_file_path
            try:
                entity_context = data_structures_utils.get_inner_dict(
                    definition_context[context_path],
                    entity_context_path,
                )
                entity_lines_range = [entity_context.get("start_line", 1), entity_context.get("end_line", 1)]
                entity_code_lines = entity_context.get("code_lines", [])
                skipped_checks = entity_context.get("skipped_checks")
            except KeyError:
                # TODO: Context info isn't working for modules
                entity_lines_range = [1, 1]
                entity_code_lines = []
                skipped_checks = None

            if full_file_path in self.evaluations_context:
                variables_evaluations = {}
                for var_name, context_info in self.evaluations_context.get(full_file_path, {}).items():
                    variables_evaluations[var_name] = dataclasses.asdict(context_info)
                entity_evaluations = BaseVariableEvaluation.reduce_entity_evaluations(
                    variables_evaluations, entity_context_path
                )
            results = registry.scan(scanned_file, entity, skipped_checks, runner_filter)
            absolute_scanned_file_path = get_abs_path(full_file_path)
            # This duplicates a call at the start of scan, but adding this here seems better than kludging with some tuple return type
            tags = get_resource_tags(entity_type, entity_config)
            if results:
                for check, check_result in results.items():
                    censored_code_lines = omit_secret_value_from_checks(
                        check=check,
                        check_result=check_result,
                        entity_code_lines=entity_code_lines,
                        entity_config=entity_config,
                        resource_attributes_to_omit=runner_filter.resource_attr_to_omit,
                    )

                    record = Record(
                        check_id=check.id,
                        bc_check_id=check.bc_id,
                        check_name=check.name,
                        check_result=check_result,
                        code_block=censored_code_lines,
                        file_path=scanned_file,
                        file_line_range=entity_lines_range,
                        resource=entity_id,
                        evaluations=entity_evaluations,
                        check_class=check.__class__.__module__,
                        file_abs_path=absolute_scanned_file_path,
                        entity_tags=tags,
                        caller_file_path=caller_file_path,
                        caller_file_line_range=caller_file_line_range,
                        severity=check.severity,
                        bc_category=check.bc_category,
                        benchmarks=check.benchmarks,
                        details=check.details,
                        definition_context_file_path=full_file_path.file_path,
                    )
                    if self.breadcrumbs:
                        entity_key = entity_id
                        breadcrumb = self.breadcrumbs.get(record.file_path, {}).get(entity_key)
                        if breadcrumb:
                            record = GraphRecord(record, breadcrumb)

                    record.set_guideline(check.guideline)
                    report.add_record(record=record)
            else:
                if block_type == "resource":
                    # resources without checks, but not existing ones
                    report.extra_resources.add(
                        ExtraResource(
                            file_abs_path=absolute_scanned_file_path,
                            file_path=scanned_file,
                            resource=entity_id,
                        )
                    )

    def get_entity_context_and_evaluations(self, entity: dict[str, Any]) -> dict[str, Any] | None:
        block_type = entity[CustomAttributes.BLOCK_TYPE]
        tf_source_module_obj = entity.get(CustomAttributes.SOURCE_MODULE_OBJECT)
        if isinstance(tf_source_module_obj, dict):
            tf_source_module_obj = TFModule.from_json(tf_source_module_obj)
        full_file_path = TFDefinitionKey(
            file_path=entity[CustomAttributes.FILE_PATH], tf_source_modules=tf_source_module_obj
        )

        definition_path = entity[CustomAttributes.BLOCK_NAME].split(".")
        entity_context_path = [block_type] + definition_path
        try:
            entity_context = self.context[full_file_path]  # type:ignore[index]  # at this point self.context is set
            for k in entity_context_path:
                if k in entity_context:
                    entity_context = entity_context[k]
                else:
                    logging.warning(f'Failed to find context for {".".join(entity_context_path)}')
                    return None
            entity_context["definition_path"] = definition_path
        except KeyError:
            logging.error(f"Did not find context for key {full_file_path}")
            return {}
        return entity_context



from __future__ import annotations

from typing import Any, TYPE_CHECKING

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.parsers.json import parse
from checkov.common.parsers.node import DictNode, ListNode
from checkov.common.runners.object_runner import Runner as ObjectRunner

if TYPE_CHECKING:
    from checkov.common.checks.base_check_registry import BaseCheckRegistry
    from checkov.common.typing import LibraryGraphConnector
    from checkov.common.runners.graph_builder.local_graph import ObjectLocalGraph
    from checkov.common.runners.graph_manager import ObjectGraphManager


class Runner(ObjectRunner):
    check_type = CheckType.JSON  # noqa: CCE003  # a static attribute

    def __init__(
        self,
        db_connector: LibraryGraphConnector | None = None,
        source: str = "json",
        graph_class: type[ObjectLocalGraph] | None = None,
        graph_manager: ObjectGraphManager | None = None,
    ) -> None:
        super().__init__(
            db_connector=db_connector,
            source=source,
            graph_class=graph_class,
            graph_manager=graph_manager,
        )
        self.file_extensions = ['.json']

    def import_registry(self) -> BaseCheckRegistry:
        from checkov.json_doc.registry import registry
        return registry

    def _parse_file(
        self, f: str, file_content: str | None = None
    ) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
        if not f.endswith(".json"):
            return None

        return parse(filename=f, file_content=file_content)

    def get_start_end_lines(self, end: int, result_config: dict[str, Any], start: int) -> tuple[int, int]:
        if not isinstance(result_config, (DictNode, ListNode)):
            # shouldn't happen
            return 0, 0

        start = result_config.start_mark.line
        end = result_config.end_mark.line
        return end, start



from __future__ import annotations

from update_checker import UpdateChecker


def check_for_update(package: str, version: str) -> str | None:
    try:
        checker = UpdateChecker()
        result = checker.check(package, version)
        return result.available_version
    except Exception:  # nosec
        return None



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck


class PostgreSQLServerConnectionThrottlingEnabled(BaseResourceCheck):
    def __init__(self) -> None:
        # https://docs.microsoft.com/en-us/azure/templates/microsoft.dbforpostgresql/servers
        # https://docs.microsoft.com/en-us/azure/templates/microsoft.dbforpostgresql/servers/configurations
        name = "Ensure server parameter 'connection_throttling' is set to 'ON' for PostgreSQL Database Server"
        id = "CKV_AZURE_32"
        supported_resources = ('Microsoft.DBforPostgreSQL/servers/configurations', 'configurations')
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        if "type" in conf:
            if conf["type"] == "Microsoft.DBforPostgreSQL/servers/configurations":
                if "name" in conf and conf["name"] == "connection_throttling":
                    if "properties" in conf:
                        if "value" in conf["properties"] and \
                                conf["properties"]["value"].lower() == "on":
                            return CheckResult.PASSED
                    return CheckResult.FAILED
                # If name not connection_throttling - don't report (neither pass nor fail)
            elif conf["type"] == "configurations":
                if "name" in conf and conf["name"] == "connection_throttling":
                    if "parent_type" in conf:
                        if conf["parent_type"] == "Microsoft.DBforPostgreSQL/servers":
                            if "properties" in conf:
                                if "value" in conf["properties"] and \
                                        conf["properties"]["value"].lower() == "on":
                                    return CheckResult.PASSED
                    return CheckResult.FAILED
                # If name not connection_throttling - don't report (neither pass nor fail)
        else:
            return CheckResult.FAILED


check = PostgreSQLServerConnectionThrottlingEnabled()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class GoogleCloudPostgreSqlLogDisconnection(BaseResourceCheck):
    def __init__(self):
        name = "Ensure PostgreSQL database 'log_disconnections' flag is set to 'on'"
        check_id = "CKV_GCP_53"
        supported_resources = ['google_sql_database_instance']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=check_id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for google_sql_database_instance which allows log disconnections on PostgreSQL DBs::
            :param
            conf: google_sql_database_instance
            configuration
            :return: < CheckResult >
        """
        if 'database_version' in conf.keys() and isinstance(conf['database_version'][0], str) and 'POSTGRES' in conf['database_version'][0]:
            if 'settings' in conf.keys():
                self.evaluated_keys = ['database_version/[0]/POSTGRES', 'settings']
                flags = conf['settings'][0].get('database_flags')
                if flags:
                    evaluated_keys_prefix = 'settings/[0]/database_flags'
                    if isinstance(flags[0], list):
                        # treating use cases of the following database_flags parsing
                        # (list of list of dictionaries with strings):'database_flags':
                        # [[{'name': '<key>', 'value': '<value>'}, {'name': '<key>', 'value': '<value>'}]]
                        flags = flags[0]
                        evaluated_keys_prefix += '/[0]'
                    else:
                        # treating use cases of the following database_flags parsing
                        # (list of dictionaries with arrays): 'database_flags':
                        # [{'name': ['<key>'], 'value': ['<value>']},{'name': ['<key>'], 'value': ['<value>']}]
                        flags = [{key: flag[key][0] for key in flag if key in ['name', 'value']} for flag in flags]
                    for flag in flags:
                        if (isinstance(flag, dict) and flag['name'] == 'log_disconnections') and (flag['value'] == 'on'):  # Must be explicitly set for check to pass
                            self.evaluated_keys = ['database_version/[0]/POSTGRES',
                                                   f'{evaluated_keys_prefix}/[{flags.index(flag)}]/name',
                                                   f'{evaluated_keys_prefix}/[{flags.index(flag)}]/value']
                            return CheckResult.PASSED
                    self.evaluated_keys = ['database_version/[0]/POSTGRES', 'settings/[0]/database_flags']

            return CheckResult.FAILED
        return CheckResult.UNKNOWN


check = GoogleCloudPostgreSqlLogDisconnection()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class IMDSv1Disabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure Instance Metadata Service Version 1 is not enabled"
        id = "CKV_AWS_79"
        categories = (CheckCategories.GENERAL_SECURITY,)
        supported_resources = ("aws_instance", "aws_launch_template", "aws_launch_configuration")
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        """
        Looks for if the metadata service is disabled or requires session tokens:
        https://registry.terraform.io/providers/hashicorp/aws/latest/docs/resources/instance#metadata-options
        or
        https://registry.terraform.io/providers/hashicorp/aws/latest/docs/resources/launch_template#metadata-options

        :param conf: dict of supported resource configuration
        :return: <CheckResult>
        """
        metadata_options = conf.get("metadata_options")
        if not metadata_options or not isinstance(metadata_options[0], dict):
            return CheckResult.FAILED

        if metadata_options[0].get("http_endpoint") == ["disabled"]:
            return CheckResult.PASSED

        return super().scan_resource_conf(conf)

    def get_inspected_key(self) -> str:
        return "metadata_options/[0]/http_tokens"

    def get_expected_value(self) -> Any:
        return "required"


check = IMDSv1Disabled()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck


class AKSNetworkPolicy(BaseResourceCheck):
    def __init__(self) -> None:
        # apiVersion 2017-08-03 = Fail - No networkProfile option to configure
        name = "Ensure AKS cluster has Network Policy configured"
        id = "CKV_AZURE_7"
        supported_resources = ('Microsoft.ContainerService/managedClusters',)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        if "apiVersion" in conf:
            if conf["apiVersion"] == "2017-08-31":
                # No networkProfile option to configure
                return CheckResult.FAILED

        properties = conf.get('properties')
        if not properties or not isinstance(properties, dict):
            return CheckResult.FAILED
        network_profile = properties.get('networkProfile')
        if not network_profile:
            return CheckResult.FAILED
        network_policy = network_profile.get('networkPolicy')
        if network_policy:
            return CheckResult.PASSED
        return CheckResult.FAILED


check = AKSNetworkPolicy()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class AzureSearchAllowedIPsNotGlobal(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        # Setting the allowed ips to include global routes CIDR - 0.0.0.0/0 makes the resource public
        name = "Ensure Azure Cognitive Search service allowed IPS does not give public Access"
        id = "CKV_AZURE_210"
        supported_resources = ("azurerm_search_service",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "allowed_ips"

    def get_forbidden_values(self) -> list[Any]:
        return ["0.0.0.0/0"]


check = AzureSearchAllowedIPsNotGlobal()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class ECSClusterContainerInsights(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure container insights are enabled on ECS cluster"
        id = "CKV_AWS_65"
        supported_resources = ("AWS::ECS::Cluster",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        """
            Looks for container insights configuration on ECS cluster:
            https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-cluster.html#cfn-ecs-cluster-clustersettings
        :param conf: AWS::ECS::Cluster configuration
        :return: <CheckResult>
        """
        properties = conf.get("Properties")
        if properties and isinstance(properties, dict):
            settings = properties.get("ClusterSettings")
            if settings and isinstance(settings, list):
                for setting in settings:
                    if setting["Name"] == "containerInsights" and setting["Value"] == "enabled":
                        return CheckResult.PASSED
        return CheckResult.FAILED


check = ECSClusterContainerInsights()



from __future__ import annotations

import os.path
from typing import Dict, Set, Any
import re
import json
import os


MODULE_EXPORTS_PATTERN = r'module\.exports\s*=\s*({.*?});'
EXPORT_DEFAULT_PATTERN = r'export\s*default\s*({.*?});'


def _parse_export(file_content: str, pattern: str) -> Dict[str, Any] | None:
    module_export_match = re.search(pattern, file_content, re.DOTALL)

    if module_export_match:
        module_exports_str = module_export_match.group(1)
        # for having for all the keys and values double quotes and removing spaces
        module_exports_str = re.sub(r'\s+', '', re.sub(r'([{\s,])(\w+):', r'\1"\2":', module_exports_str)
                                    .replace("'", "\""))
        module_exports: Dict[str, Any] = json.loads(module_exports_str)
        return module_exports
    return None


def parse_webpack_file(file_content: str, relevant_packages: Set[str]) -> Dict[str, Any]:
    output: Dict[str, Any] = {"packageAliases": {}}
    module_exports_json = _parse_export(file_content, MODULE_EXPORTS_PATTERN)
    if module_exports_json:
        aliases = module_exports_json.get("resolve", {}).get("alias", {})
        for imported_name in aliases:
            package_name = aliases[imported_name]
            if package_name in relevant_packages:
                output["packageAliases"].setdefault(package_name, {"packageAliases": []})["packageAliases"].append(imported_name)
    return output


def parse_tsconfig_file(file_content: str, relevant_packages: Set[str]) -> Dict[str, Any]:
    output: Dict[str, Any] = {"packageAliases": {}}
    tsconfig_json = json.loads(file_content)
    paths = tsconfig_json.get("compilerOptions", {}).get("paths", {})
    for imported_name in paths:
        for package_relative_path in paths[imported_name]:
            package_name = os.path.basename(package_relative_path)
            if package_name in relevant_packages:
                output["packageAliases"].setdefault(package_name, {"packageAliases": []})["packageAliases"].append(imported_name)
    return output


def parse_babel_file(file_content: str, relevant_packages: Set[str]) -> Dict[str, Any]:
    output: Dict[str, Any] = {"packageAliases": {}}
    babelrc_json = json.loads(file_content)
    plugins = babelrc_json.get("plugins", {})
    for plugin in plugins:
        if len(plugin) > 1:
            plugin_object = plugin[1]
            aliases = plugin_object.get("alias", {})
            for imported_name in aliases:
                package_name = aliases[imported_name]
                if package_name in relevant_packages:
                    output["packageAliases"].setdefault(package_name, {"packageAliases": []})["packageAliases"].append(imported_name)
    return output


def parse_rollup_file(file_content: str, relevant_packages: Set[str]) -> Dict[str, Any]:
    output: Dict[str, Any] = {"packageAliases": {}}
    export_default_match = re.search(EXPORT_DEFAULT_PATTERN, file_content, re.DOTALL)
    if export_default_match:
        export_default_str = export_default_match.group(1)
        # for having for all the keys and values doube quotes and removing spaces
        export_default_str = re.sub(r'\s+', '', re.sub(r'([{\s,])(\w+):', r'\1"\2":', export_default_str)
                                    .replace("'", "\""))

        # Defining a regular expression pattern to match the elements within the "plugins" list
        pattern = r'alias\(\{[^)]*\}\)'
        matches = re.findall(pattern, export_default_str)

        for alias_object_str in matches:
            alias_object = json.loads(alias_object_str[6:-1])  # removing 'alias(' and ')'
            for entry in alias_object.get("entries", []):
                package_name = entry["replacement"]
                if entry["replacement"] in relevant_packages:
                    imported_name = entry["find"]
                    output["packageAliases"].setdefault(package_name, {"packageAliases": []})["packageAliases"].append(imported_name)
    return output


def parse_package_json_file(file_content: str, relevant_packages: Set[str]) -> Dict[str, Any]:
    output: Dict[str, Any] = {"packageAliases": {}}
    package_json = json.loads(file_content)
    aliases: Dict[str, str] = dict()
    if "alias" in package_json:
        aliases.update(package_json["alias"])
    if package_json.get("aliasify", {}).get("aliases"):
        aliases.update(package_json["aliasify"]["aliases"])
    for imported_name in aliases:
        package_name = aliases[imported_name]
        if package_name in relevant_packages:
            output["packageAliases"].setdefault(package_name, {"packageAliases": []})["packageAliases"].append(imported_name)
    return output


def parse_snowpack_file(file_content: str, relevant_packages: Set[str]) -> Dict[str, Any]:
    output: Dict[str, Any] = {"packageAliases": {}}
    module_exports_json = _parse_export(file_content, MODULE_EXPORTS_PATTERN)
    if module_exports_json:
        aliases = module_exports_json.get("alias", {})
        for imported_name in aliases:
            package_name = aliases[imported_name]
            if package_name in relevant_packages:
                if package_name in relevant_packages:
                    output["packageAliases"].setdefault(package_name, {"packageAliases": []})["packageAliases"].append(imported_name)
    return output


def parse_vite_file(file_content: str, relevant_packages: Set[str]) -> Dict[str, Any]:
    output: Dict[str, Any] = {"packageAliases": {}}
    export_default_match = _parse_export(file_content, EXPORT_DEFAULT_PATTERN)
    if export_default_match:
        aliases = export_default_match.get("resolve", {}).get("alias", {})
        for imported_name in aliases:
            package_name = aliases[imported_name]
            if package_name in relevant_packages:
                output["packageAliases"].setdefault(package_name, {"packageAliases": []})["packageAliases"].append(imported_name)
    return output



from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class WebhookInsecureSsl(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure GitHub repository webhooks are using HTTPS"
        id = "CKV_GIT_2"
        supported_resources = ("github_repository_webhook",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self) -> str:
        return "configuration/[0]/insecure_ssl/[0]"

    def get_expected_value(self) -> Any:
        return False


check = WebhookInsecureSsl()



from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class CloudfrontDistributionDefaultRoot(BaseResourceValueCheck):

    def __init__(self):
        """
        NIST.800-53.r5 SC-7(11), NIST.800-53.r5 SC-7(16)
        CloudFront distributions should have a default root object configured
        """
        name = "Ensure Cloudfront distribution has a default root object configured"
        id = "CKV_AWS_305"
        supported_resources = ['aws_cloudfront_distribution']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "default_root_object"

    def get_expected_value(self):
        return ANY_VALUE


check = CloudfrontDistributionDefaultRoot()



from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class CloudFunctionsShouldNotBePublic(BaseResourceCheck):
    def __init__(self):
        name = "Cloud functions should not be public"
        id = "CKV_GCP_107"
        supported_resources = [
            "google_cloudfunctions_function_iam_member",
            "google_cloudfunctions_function_iam_binding",
            "google_cloudfunctions2_function_iam_member",
            "google_cloudfunctions2_function_iam_binding"
        ]
        categories = [CheckCategories.APPLICATION_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        if conf.get("member") and isinstance(conf.get("member"), list):
            if conf.get("member") == ["allUsers"]:
                return CheckResult.FAILED
            return CheckResult.PASSED
        if conf.get("members") and isinstance(conf.get("members")[0], list):
            if "allUsers" in conf.get("members")[0]:
                return CheckResult.FAILED
            return CheckResult.PASSED
        return CheckResult.UNKNOWN


check = CloudFunctionsShouldNotBePublic()



from __future__ import annotations

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class SQLDatabaseLedgerEnabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        """
        Ledger helps protect data from any attacker or high-privileged user, including database administrators (DBAs),
        system administrators, and cloud administrators. As with a traditional ledger, the feature preserves
        historical data.
        If a row is updated in the database, its previous value is maintained and protected
        in a history table. Ledger provides a chronicle of all changes made to the database over time.
        Ledger and the historical data are managed transparently, offering protection without any application changes.
        The feature maintains historical data in a relational form to support SQL queries for auditing,
        forensics, and other purposes.
        It provides guarantees of cryptographic data integrity while maintaining the power, flexibility,
        and performance of the SQL database.

        Note that:
        - Ledger needs to be enabled at the deployment of the database and can't be removed once enabled
        - Ledger may come with performance impact, which means that it is advise to closely monitor
          the database performance in order to ensure that the database meets the performance objectives
        - Ledger comes with an additional cost, due to the data being stored

        """
        name = "Ensure that the Ledger feature is enabled on database that "
        name += "requires cryptographic proof and nonrepudiation of data integrity"
        id = "CKV_AZURE_224"
        supported_resources = ("azurerm_mssql_database",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "ledger_enabled"


check = SQLDatabaseLedgerEnabled()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class CosmosDBLocalAuthDisabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        # This is the full description of your check
        description = "Ensure that Local Authentication is disabled on CosmosDB"

        # This is the Unique ID for your check
        id = "CKV_AZURE_140"

        # These are the terraform objects supported by this check (ex: aws_iam_policy_document)
        supported_resources = ('azurerm_cosmosdb_account',)

        # Valid CheckCategories are defined in checkov/common/models/enums.py
        categories = (CheckCategories.IAM,)
        super().__init__(name=description, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if conf.get("kind") == ["GlobalDocumentDB"]:
            return super().scan_resource_conf(conf)
        return CheckResult.UNKNOWN

    def get_inspected_key(self) -> str:
        return "local_authentication_disabled"

    def get_expected_value(self) -> Any:
        return True


check = CosmosDBLocalAuthDisabled()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class AzureInstancePassword(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Azure Instance does not use basic authentication(Use SSH Key Instead)"
        id = "CKV_AZURE_1"
        supported_resources = ['azurerm_virtual_machine', 'azurerm_linux_virtual_machine']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for password configuration at azure_instance:
            https://www.terraform.io/docs/providers/azure/r/instance.html
        :param conf: azure_instance configuration
        :return: <CheckResult>
        """
        if 'os_profile_linux_config' in conf:
            linux_config = conf['os_profile_linux_config'][0]
            if isinstance(linux_config, dict) and 'disable_password_authentication' in linux_config:
                disable_password_authentication = linux_config['disable_password_authentication']
                self.evaluated_keys = ['os_profile_linux_config/[0]/disable_password_authentication']
                if disable_password_authentication == [False]:
                    return CheckResult.FAILED
            else:
                self.evaluated_keys = ['os_profile_linux_config']

        if not conf.get('disable_password_authentication', [True])[0]:
            self.evaluated_keys.append('disable_password_authentication')
            return CheckResult.FAILED

        return CheckResult.PASSED


check = AzureInstancePassword()



from typing import Any, Dict


class Edge:
    __slots__ = ("dest", "label", "origin")

    def __init__(self, origin: int, dest: int, label: str) -> None:
        self.origin = origin
        self.dest = dest
        self.label = label

    def __str__(self) -> str:
        return f"[{self.origin} -({self.label})-> {self.dest}]"

    def __eq__(self, other: Any) -> bool:
        return isinstance(other, Edge) and str(self) == str(other)

    def __ne__(self, other: Any) -> bool:
        return not self.__eq__(other)

    def __hash__(self) -> int:
        return hash(str(self))

    def to_dict(self) -> Dict[str, Any]:
        return {
            'origin': self.origin,
            'dest': self.dest,
            'label': self.label
        }



from __future__ import annotations

import os
import sys
from typing import NoReturn

from colorama import Fore
from tqdm import tqdm

from checkov.common.util.type_forcers import convert_str_to_bool

DEFAULT_BAR_FORMAT = f'{{l_bar}}{Fore.WHITE}{{bar:20}}{Fore.RESET}|[{{n_fmt}}/{{total_fmt}}]{{postfix}}'
LOGS_ENABLED = os.getenv('LOG_LEVEL', False)
RUN_IN_DOCKER = convert_str_to_bool(os.getenv("RUN_IN_DOCKER", "False"))


class ProgressBar:
    def __init__(self, framework: str) -> None:
        self.pbar: tqdm[NoReturn] | None = None
        self.framework = framework
        self.is_off = not self.should_show_progress_bar()

    def initiate(self, total: int) -> None:
        if total <= 0:
            self.is_off = True

        if self.is_off:
            return

        if self.pbar is not None:
            self.pbar.reset(total)
        else:
            self.pbar = tqdm(total=total,
                             bar_format=DEFAULT_BAR_FORMAT,
                             desc=f'[ {self.framework} framework ]')

    def update(self, value: int = 1) -> None:
        if self.is_off:
            return
        if not self.pbar:
            raise AttributeError('Progress bar was not initiated, cannot update')

        self.pbar.update(value)

    def set_description(self, desc: str) -> None:
        if self.is_off:
            return
        if not self.pbar:
            raise AttributeError('Progress bar was not initiated, cannot set description')

        self.pbar.set_description(desc=desc)

    def close(self) -> None:
        if self.is_off:
            return
        if not self.pbar:
            raise AttributeError('Progress bar was not initiated, cannot close')

        self.pbar.close()

    def set_additional_data(self, data: dict[str, str]) -> None:
        if self.is_off:
            return
        if not self.pbar:
            raise AttributeError('Progress bar was not initiated, cannot set additional data')

        self.pbar.set_postfix(data)

    def turn_off_progress_bar(self) -> None:
        self.is_off = True

    @staticmethod
    def should_show_progress_bar() -> bool:
        if all([not LOGS_ENABLED, not RUN_IN_DOCKER, sys.__stdout__.isatty()]):
            return True
        return False



from typing import Dict, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck

DISALLOWED_MEMBERS = {"allUsers", "allAuthenticatedUsers"}


class GoogleKMSKeyIsPublic(BaseResourceCheck):
    def __init__(self) -> None:
        """
        ensure key is not publicly available
        KMS Crypto Key policy should not set 'allUsers' or 'allAuthenticatedUsers' in the attribute 'member'/'members'
        """
        name = "Esnure KMS policy should not allow public access"
        id = "CKV_GCP_112"
        supported_resources = (
            "google_kms_crypto_key_iam_policy",
            "google_kms_crypto_key_iam_binding",
            "google_kms_crypto_key_iam_member",
        )
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        policy_data = conf.get("policy_data")
        if policy_data and isinstance(policy_data, list):
            for policy in policy_data:
                if not isinstance(policy, dict):
                    continue
                bindings = policy.get("bindings")
                if bindings and isinstance(bindings, list):
                    for binding in bindings:
                        members = binding.get("members")
                        if members and isinstance(members, list):
                            for member in members:
                                if member in DISALLOWED_MEMBERS:
                                    return CheckResult.FAILED
            return CheckResult.PASSED

        members = conf.get("members")
        if members and isinstance(members, list):
            for member in members[0]:
                if member in DISALLOWED_MEMBERS:
                    return CheckResult.FAILED
            return CheckResult.PASSED

        member = conf.get("member")
        if member and isinstance(member, list):
            if member[0] in DISALLOWED_MEMBERS:
                return CheckResult.FAILED
            return CheckResult.PASSED

        return CheckResult.UNKNOWN


check = GoogleKMSKeyIsPublic()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from typing import List


class AppServiceDotnetFrameworkVersion(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that 'Net Framework' version is the latest, if used as a part of the web app"
        id = "CKV_AZURE_80"
        supported_resources = ['azurerm_app_service', 'azurerm_windows_web_app']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        if conf.get('site_config') and isinstance(conf.get('site_config'), list):
            site_config = conf.get('site_config')[0]
            if site_config.get('dotnet_framework_version') and isinstance(site_config.get('dotnet_framework_version'), list):
                if site_config.get('dotnet_framework_version')[0] == "v6.0":
                    return CheckResult.PASSED
                return CheckResult.FAILED
            if site_config.get('application_stack') and isinstance(site_config.get('application_stack'), list):
                stack = site_config.get('application_stack')[0]
                if stack.get('dotnet_version') and isinstance(stack.get('dotnet_version'), list):
                    if stack.get('dotnet_version')[0] == "v7.0":
                        return CheckResult.PASSED
                    return CheckResult.FAILED

        return CheckResult.UNKNOWN

    def get_expected_values(self) -> List[str]:
        return ["v6.0", "v7.0"]


check = AppServiceDotnetFrameworkVersion()



from typing import Dict, Any, List

from hcl2 import START_LINE, END_LINE

from checkov.terraform.context_parsers.base_parser import BaseContextParser
import dpath


class LocalsContextParser(BaseContextParser):
    def __init__(self) -> None:
        definition_type = "locals"
        super().__init__(definition_type=definition_type)

    def _collect_local_values(self, local_block: Dict[str, Any]) -> None:
        for local_name, local_value in local_block.items():
            if local_name in {START_LINE, END_LINE}:
                continue

            local_value = local_value[0] if isinstance(local_value, list) and len(local_value) > 0 else local_value
            if type(local_value) in (int, float, bool, str, dict):
                dpath.new(self.context, ["assignments", local_name], local_value)

    def get_entity_context_path(self, entity_block: Dict[str, Dict[str, Any]]) -> List[str]:
        return []

    def enrich_definition_block(self, definition_blocks: List[Dict[str, Any]]) -> Dict[str, Any]:
        for entity_block in definition_blocks:
            if START_LINE in entity_block.keys():
                self.context["start_line"] = entity_block[START_LINE]
            if END_LINE in entity_block.keys():
                self.context["end_line"] = entity_block[END_LINE]
            if "start_line" in self.context and "end_line" in self.context:
                self.context["code_lines"] = self.file_lines[self.context["start_line"] - 1: self.context["end_line"]]

            if isinstance(entity_block, dict):
                self._collect_local_values(entity_block)
        return self.context


parser = LocalsContextParser()



from __future__ import annotations

from typing import Any

from bc_jsonpath_ng import parse

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.github.base_github_configuration_check import BaseGithubCheck
from checkov.github.schemas.org_security import schema as org_security_schema
from checkov.json_doc.enums import BlockType


class GithubSSO(BaseGithubCheck):
    def __init__(self) -> None:
        name = "Ensure GitHub organization security settings require SSO"
        id = "CKV_GITHUB_2"
        categories = (CheckCategories.SUPPLY_CHAIN, )
        super().__init__(
            id=id,
            name=name,
            categories=categories,
            supported_entities=["*"],
            block_type=BlockType.DOCUMENT
        )

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> CheckResult:  # type:ignore[override]
        if org_security_schema.validate(conf):
            jsonpath_expression = parse("$..{}".format(self.get_evaluated_keys()[0].replace("/", ".")))
            if len(jsonpath_expression.find(conf)) > 0:
                return CheckResult.PASSED
            else:
                return CheckResult.FAILED
        return CheckResult.UNKNOWN

    def get_evaluated_keys(self) -> list[str]:
        return ['data/organization/samlIdentityProvider/ssoUrl']


check = GithubSSO()



from __future__ import annotations

from typing import Any
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.checks.enums import BlockType
from checkov.common.util.consts import LINE_FIELD_NAMES
from checkov.openapi.checks.base_openapi_check import BaseOpenapiCheck


class ClearTestAPIKey(BaseOpenapiCheck):
    def __init__(self) -> None:
        id = "CKV_OPENAPI_20"
        name = "Ensure that API keys are not sent over cleartext"
        categories = (CheckCategories.API_SECURITY,)
        supported_resources = ('paths',)
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_resources,
                         block_type=BlockType.DOCUMENT)

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]]:  # type:ignore[override]  # return type is different than the base class
        schemes = conf.get("schemes")
        if schemes and isinstance(schemes, list):
            if "http" not in schemes and "ws" not in schemes:
                return CheckResult.PASSED, conf

        servers = conf.get("servers")
        if servers and isinstance(servers, list):
            if not any(server['url'].startswith('http://') for server in servers) and \
               not any(server['url'].startswith('ws://') for server in servers):
                return CheckResult.PASSED, conf

        components = conf.get("components")
        security_def = conf.get("securityDefinitions")
        if components and isinstance(components, dict):
            security_schemes = components.get("securitySchemes") or {}
        elif security_def:
            security_schemes = security_def
        else:
            return CheckResult.PASSED, conf

        paths = conf.get('paths')
        if not isinstance(paths, dict):
            return CheckResult.PASSED, security_schemes

        filtered_dict = {}
        if isinstance(security_schemes, dict):
            for name, scheme in security_schemes.items():
                if isinstance(scheme, dict) and scheme.get('type') == "apiKey":
                    filtered_dict[name] = scheme

        if not filtered_dict:
            return CheckResult.PASSED, security_schemes

        for key, path in paths.items():
            if not path:
                continue
            if key in LINE_FIELD_NAMES:
                continue
            for value in path.values():
                if not isinstance(value, dict):
                    continue
                operation_security = value.get('security')
                if operation_security and isinstance(operation_security, list):
                    for sec in operation_security[0]:
                        if sec in filtered_dict:
                            return CheckResult.FAILED, security_schemes

        return CheckResult.PASSED, conf


check = ClearTestAPIKey()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class CloudBuildWorkersArePrivate(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure Cloud build workers are private"
        id = "CKV_GCP_86"
        supported_resources = ['google_cloudbuild_worker_pool']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "worker_config/[0]/no_external_ip"


check = CloudBuildWorkersArePrivate()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from typing import Any


class AKSEphemeralOSDisks(BaseResourceValueCheck):
    def __init__(self) -> None:
        """
        Temporary data can contain sensitive data at some points, by using ephemeral disks,
        we ensure that data written to OS disk is stored on local VM storage and isn't persisted to Azure Storage

        Azure automatically replicates data stored in the managed OS disk of a virtual machine to Azure storage
        to avoid data loss in case the virtual machine needs to be relocated to another host.
        Generally speaking, containers are not designed to have local state persisted to the managed OS disk,
        hence this behavior offers limited value to AKS hosted while providing some drawbacks,
        including slower node provisioning and higher read/write latency.

        Ephemeral disks allow us also to have faster cluster operations like scale or upgrade
        due to faster re-imaging and boot times.
        """
        name = "Ensure ephemeral disks are used for OS disks"
        id = "CKV_AZURE_226"
        supported_resources = ("azurerm_kubernetes_cluster",)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "os_disk_type"

    def get_expected_value(self) -> Any:
        return "Ephemeral"


check = AKSEphemeralOSDisks()



from typing import Dict, List, Any
import re
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.type_forcers import force_list
from checkov.terraform.checks.data.base_check import BaseDataCheck

gh_repo_regex = re.compile(r'repo:[^/]+/[^/]+')


class GithubActionsOIDCTrustPolicy(BaseDataCheck):
    def __init__(self):
        name = 'Ensure GitHub Actions OIDC trust policies only allows actions from a specific known organization'
        id = "CKV_AWS_358"
        supported_data = ("aws_iam_policy_document",)
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_data=supported_data)

    def scan_data_conf(self, conf: Dict[str, List[Any]], entity_type: str) -> CheckResult:
        statements = force_list(conf.get('statement'))
        for statement in statements:
            found_federated_gh_oidc = False
            if isinstance(statement, dict):
                if statement.get('principals'):
                    principals = statement['principals']
                    for principal in force_list(principals):
                        if 'type' not in principal and 'identifiers' not in principal:
                            continue
                        principal_type = principal['type']
                        principal_identifiers = principal['identifiers']
                        if isinstance(principal_type, list) and len(
                                principal_type) and 'Federated' in principal_type and isinstance(principal_identifiers,
                                                                                                 list):
                            for identifier in principal_identifiers:
                                if isinstance(identifier,
                                              list) and 'oidc-provider/token.actions.githubusercontent.com' in \
                                        identifier[0]:
                                    found_federated_gh_oidc = True
                                    break
                if not found_federated_gh_oidc:
                    return CheckResult.PASSED
                if found_federated_gh_oidc and not statement.get('condition'):
                    return CheckResult.FAILED
                found_sub_condition_variable = False
                found_sub_condition_value = False
                for condition in statement.get('condition'):
                    condition_variables = condition.get('variable')
                    condition_values = condition.get('values')
                    if isinstance(condition_variables, list):
                        for condition_variable in condition_variables:
                            if condition_variable == 'token.actions.githubusercontent.com:sub':
                                found_sub_condition_variable = True
                                break
                        for condition_value in condition_values:
                            if isinstance(condition_value, list) and gh_repo_regex.search(condition_value[0]):
                                found_sub_condition_value = True
                                break
                        if found_sub_condition_value and found_sub_condition_variable:
                            return CheckResult.PASSED

                # Found a federated GitHub user, but no restirctions
                return CheckResult.FAILED

        return CheckResult.PASSED


check = GithubActionsOIDCTrustPolicy()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class TLSPoliciesAreSecure(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Alibaba Cloud Cypher Policy are secure"
        id = "CKV_ALI_33"
        supported_resources = ['alicloud_slb_tls_cipher_policy']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'tls_versions'

    def get_forbidden_values(self):
        return ["TLSv1.1", "TLSv1.0"]


check = TLSPoliciesAreSecure()



from __future__ import annotations

import asyncio
import logging
import os
from collections.abc import Sequence, Iterable
from pathlib import Path
from typing import Any, TYPE_CHECKING

from aiomultiprocess import Pool  # type:ignore[import]

from checkov.common.bridgecrew.vulnerability_scanning.integrations.package_scanning import PackageScanningIntegration

if TYPE_CHECKING:
    from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration


async def _report_results_to_bridgecrew_async(
    scan_results: Iterable[dict[str, Any]],
    bc_integration: BcPlatformIntegration,
    bc_api_key: str,
) -> Sequence[int]:
    package_scanning_int = PackageScanningIntegration()
    args = [(result, bc_integration, bc_api_key, Path(result["repository"])) for result in scan_results]

    if os.getenv("PYCHARM_HOSTED") == "1":
        # PYCHARM_HOSTED env variable equals 1 when running via Pycharm.
        # it avoids us from crashing, which happens when using multiprocessing via Pycharm's debug-mode
        logging.warning("reporting the results in sequence for avoiding crashing when running via Pycharm")
        exit_codes = []
        for curr_arg in args:
            exit_codes.append(await package_scanning_int.report_results_async(*curr_arg))
    else:
        async with Pool() as pool:
            exit_codes = await pool.starmap(package_scanning_int.report_results_async, args)

    return exit_codes


def report_results_to_bridgecrew(
    scan_results: Iterable[dict[str, Any]],
    bc_integration: BcPlatformIntegration,
    bc_api_key: str,
) -> Sequence[int]:
    return asyncio.run(_report_results_to_bridgecrew_async(scan_results, bc_integration, bc_api_key))



from __future__ import annotations

import os
import platform
import re
import logging
from typing import TYPE_CHECKING

from checkov.terraform.module_loading.content import ModuleContent
from checkov.terraform.module_loading.loader import ModuleLoader

if TYPE_CHECKING:
    from checkov.terraform.module_loading.module_params import ModuleParams

WINDOWS_MODULE_SOURCE_PATH_PATTERN = re.compile("[a-zA-Z]:\\\\")


class LocalPathLoader(ModuleLoader):
    def __init__(self) -> None:
        super().__init__()
        self.is_external = False

    def discover(self, module_params: ModuleParams) -> None:
        pass

    def _is_matching_loader(self, module_params: ModuleParams) -> bool:
        if module_params.module_source.startswith(("./", "../", module_params.current_dir, "/")):
            return True

        if platform.system() == "Windows":
            logging.debug("Platform: Windows")
            if re.match(WINDOWS_MODULE_SOURCE_PATH_PATTERN, module_params.module_source):
                return True

        return False

    def _load_module(self, module_params: ModuleParams) -> ModuleContent:
        module_path = os.path.normpath(os.path.join(module_params.current_dir, module_params.module_source))
        if module_params.module_source.startswith(module_params.current_dir):
            module_path = module_params.module_source
        if not os.path.exists(module_path):
            raise FileNotFoundError(module_path)

        return ModuleContent(module_path)

    def _find_module_path(self, module_params: ModuleParams) -> str:
        # to determine the exact path here would mimic _load_module()
        return ""


loader = LocalPathLoader()



import logging
from typing import Dict, Any, List

import hcl2
from hcl2 import START_LINE, END_LINE

from checkov.terraform.context_parsers.base_parser import BaseContextParser


class ProviderContextParser(BaseContextParser):
    def __init__(self) -> None:
        definition_type = "provider"
        super().__init__(definition_type=definition_type)

    def get_entity_context_path(self, entity_block: Dict[str, Dict[str, Any]]) -> List[str]:
        entity_type, entity_value = next(iter(entity_block.items()))
        return [entity_type, entity_value.get("alias", ["default"])[0]]

    def get_entity_definition_path(self, entity_block: Dict[str, Dict[str, Any]]) -> List[str]:
        entity_type, _ = next(iter(entity_block.items()))
        return [entity_type]

    def enrich_definition_block(self, definition_blocks: List[Dict[str, Any]]) -> Dict[str, Any]:
        for entity_block in definition_blocks:
            entity_type, entity_config = next(iter(entity_block.items()))
            entity_name = entity_config.get("alias", ["default"])[0]
            self.context[entity_type][entity_name] = {
                "start_line": entity_config[START_LINE],
                "end_line": entity_config[END_LINE],
                "code_lines": self.file_lines[entity_config[START_LINE] - 1: entity_config[END_LINE]],
            }

        return self.context

    def _is_block_signature(self, line_num: int, line_tokens: List[str], entity_context_path: List[str]) -> bool:
        # Ignore the alias as it is not part of the signature
        is_provider = super()._is_block_signature(line_num, line_tokens, entity_context_path[0:-1])
        if not is_provider or "=" in line_tokens or line_tokens[0] != "provider":
            if not all(bracket in line_tokens for bracket in ("{", "}")):
                # The line provider = alias is not a provider block although it has the correct words
                # Also skips comments that include words like provider and aws
                return False

        end_line = self._compute_definition_end_line(line_num)
        provider_type = entity_context_path[0]
        try:
            provider_obj = hcl2.loads(
                "\n".join(
                    map(lambda obj: obj[1], self.file_lines[line_num - 1 : end_line if end_line > line_num else line_num])
                )
            )["provider"][0]
        except Exception as e:
            logging.info(f'got exception while loading file {self.tf_file}\n {e}')
            return False
        alias = provider_obj[provider_type].get("alias", ["default"])
        return super()._is_block_signature(line_num, line_tokens + alias, entity_context_path)


parser = ProviderContextParser()



from __future__ import annotations

from typing import Any

from checkov.common.util.type_forcers import force_int
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class SecretManagerSecret90days(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Secrets Manager secrets should be rotated within 90 days"
        id = "CKV_AWS_304"
        supported_resources = ("aws_secretsmanager_secret_rotation",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        rules = conf.get("rotation_rules")
        if rules and isinstance(rules, list):
            days = rules[0].get('automatically_after_days')
            if days and isinstance(days, list):
                days = force_int(days[0])
                if days is not None and days < 90:
                    return CheckResult.PASSED
        return CheckResult.FAILED


check = SecretManagerSecret90days()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class GoogleComputeSSLPolicy(BaseResourceCheck):
    def __init__(self):
        name = "Ensure no HTTPS or SSL proxy load balancers permit SSL policies with weak cipher suites"
        id = "CKV_GCP_4"
        supported_resources = ['google_compute_ssl_policy']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            https://www.terraform.io/docs/providers/google/r/compute_ssl_policy.html
        :param conf: google_compute_ssl_policy configuration
        :return: <CheckResult>
        """
        if 'profile' in conf.keys():
            self.evaluated_keys = ['profile']
            if conf['profile'][0] == 'RESTRICTED':
                return CheckResult.PASSED
            elif conf['profile'][0] == 'MODERN':
                if 'min_tls_version' in conf.keys():
                    self.evaluated_keys.append('min_tls_version')
                    if conf['min_tls_version'][0] == "TLS_1_2":
                        return CheckResult.PASSED
            elif conf['profile'][0] == 'CUSTOM':
                self.evaluated_keys.append('custom_features')
                if not any(item in conf['custom_features'][0] for item in ['TLS_RSA_WITH_AES_128_GCM_SHA256',
                                                                           'TLS_RSA_WITH_AES_256_GCM_SHA384',
                                                                           'TLS_RSA_WITH_AES_128_CBC_SHA',
                                                                           'TLS_RSA_WITH_AES_256_CBC_SHA',
                                                                           'TLS_RSA_WITH_3DES_EDE_CBC_SHA']):
                    return CheckResult.PASSED
        return CheckResult.FAILED


check = GoogleComputeSSLPolicy()



from __future__ import annotations
import concurrent.futures
import logging
from typing import Any, TYPE_CHECKING

from checkov.common.graph.checks_infra import debug
from checkov.common.models.enums import CheckResult
from checkov.runner_filter import RunnerFilter

if TYPE_CHECKING:
    from checkov.common.graph.checks_infra.base_check import BaseGraphCheck
    from checkov.common.graph.checks_infra.base_parser import BaseGraphCheckParser
    from checkov.common.typing import _CheckResult, LibraryGraph


class BaseRegistry:
    def __init__(self, parser: BaseGraphCheckParser) -> None:
        self.checks: "list[BaseGraphCheck]" = []
        self.parser = parser

    def load_checks(self) -> None:
        raise NotImplementedError

    def run_checks(
        self, graph_connector: LibraryGraph, runner_filter: RunnerFilter, report_type: str
    ) -> dict[BaseGraphCheck, list[_CheckResult]]:

        check_results: "dict[BaseGraphCheck, list[_CheckResult]]" = {}

        checks_to_run = [c for c in self.checks if runner_filter.should_run_check(c, report_type=report_type)]
        with concurrent.futures.ThreadPoolExecutor() as executor:
            concurrent.futures.wait(
                [executor.submit(self.run_check_parallel, check, check_results, graph_connector)
                 for check in checks_to_run]
            )
        return check_results

    def run_check_parallel(
            self, check: BaseGraphCheck, check_results: dict[BaseGraphCheck, list[_CheckResult]],
            graph_connector: LibraryGraph
    ) -> None:
        logging.debug(f'Running graph check: {check.id}')
        debug.graph_check(check_id=check.id, check_name=check.name)

        passed, failed, unknown = check.run(graph_connector)
        evaluated_keys = check.get_evaluated_keys()
        check_result = self._process_check_result(passed, [], CheckResult.PASSED, evaluated_keys)
        check_result = self._process_check_result(failed, check_result, CheckResult.FAILED, evaluated_keys)
        check_result = self._process_check_result(unknown, check_result, CheckResult.UNKNOWN, evaluated_keys)
        check_results[check] = check_result

    @staticmethod
    def _process_check_result(
        results: list[dict[str, Any]],
        processed_results: list[_CheckResult],
        result: CheckResult,
        evaluated_keys: list[str],
    ) -> list[_CheckResult]:
        for vertex in results:
            processed_results.append({"result": result, "entity": vertex, "evaluated_keys": evaluated_keys})
        return processed_results



from __future__ import annotations

from typing import TYPE_CHECKING

from checkov.secrets.parsers.single_line_parser import BaseSingleLineParser

if TYPE_CHECKING:
    from detect_secrets.util.code_snippet import CodeSnippet


class TerraformSingleLineParser(BaseSingleLineParser):
    def ignore_secret(self, raw_context: CodeSnippet) -> bool:
        return self.ignore_terraform_data_block(raw_context=raw_context)

    def ignore_terraform_data_block(self, raw_context: CodeSnippet) -> bool:
        """Check for a possible data block usage"""

        # search backwards to find a possible 'data' block
        for line_index in range(raw_context.target_index - 1, -1, -1):
            if raw_context.lines[line_index].lstrip().startswith('data "'):
                # a data block is typically used to get remote information,
                # therefore can retrieve a secret, but has not a hardcoded secret
                return True

        return False


terraform_single_line_parser = TerraformSingleLineParser()



from checkov.terraform.checks.resource.ncp.AccessControlGroupInboundRule import AccessControlGroupInboundRule


class AccessControlGroupRuleInboundPort3389(AccessControlGroupInboundRule):
    def __init__(self):
        super().__init__(check_id="CKV_NCP_5", port=3389)


check = AccessControlGroupRuleInboundPort3389()



from __future__ import annotations

import logging
import operator
import re
from functools import reduce
from typing import Any, TYPE_CHECKING, Generator

from checkov.arm.utils import ArmElements
from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import integration as metadata_integration
from checkov.common.util.consts import LINE_FIELD_NAMES, START_LINE, END_LINE
from checkov.common.util.type_forcers import force_list

if TYPE_CHECKING:
    from checkov.common.typing import _SkippedCheck

COMMENT_REGEX = re.compile(r'([A-Z_\d]+)(:[^\n]+)?')
PARAMETERS_PATTERN = re.compile(r"\[parameters\('|'\)]")
VARIABLES_PATTERN = re.compile(r"\[variables\('|'\)]")


class ContextParser:
    """
    ARM template context parser
    """

    def __init__(self, arm_file: str, arm_template: dict[str, Any], arm_template_lines: list[tuple[int, str]]) -> None:
        self.arm_file = arm_file
        self.arm_template = arm_template
        self.arm_template_lines = arm_template_lines

    def evaluate_default_parameters(self) -> None:
        # Get parameter defaults and variable values
        parameter_defaults = {}
        if ArmElements.PARAMETERS in self.arm_template:
            for parameter, config in self.arm_template[ArmElements.PARAMETERS].items():
                if parameter in LINE_FIELD_NAMES:
                    continue
                if "defaultValue" in config:
                    parameter_defaults[parameter] = config["defaultValue"]

        variable_values = {}
        if ArmElements.VARIABLES in self.arm_template:
            for var, config in self.arm_template[ArmElements.VARIABLES].items():
                if var in LINE_FIELD_NAMES:
                    continue
                variable_values[var] = config

        # Find paths to substitute parameters and variables
        keys_w_params = self.search_deep_values('[parameters(', self.arm_template, [])
        keys_w_vars = self.search_deep_values('[variables(', self.arm_template, [])

        # Substitute Parameters and Variables
        for key_entry in keys_w_params:
            try:
                param = re.sub(
                    PARAMETERS_PATTERN,
                    "",
                    self._get_from_dict(dict(self.arm_template), key_entry[:-1])[key_entry[-1]],  # type:ignore[index]  # this will be a str
                )
                if param in parameter_defaults:
                    logging.debug(f"Replacing parameter {param} in file {self.arm_file} with default value: {parameter_defaults[param]}")
                    self._set_in_dict(dict(self.arm_template), key_entry, parameter_defaults[param])
            except TypeError:
                logging.debug(f"Failed to evaluate param in {self.arm_file}", exc_info=True)

        for key_entry in keys_w_vars:
            try:
                param = re.sub(
                    VARIABLES_PATTERN,
                    "",
                    self._get_from_dict(dict(self.arm_template), key_entry[:-1])[key_entry[-1]],  # type:ignore[index]  # this will be a str
                )
                if param in variable_values.keys():
                    self._set_in_dict(dict(self.arm_template), key_entry, variable_values[param])
                    logging.debug(
                        "Replacing variable {} in file {} with default value: {}".format(param, self.arm_file,
                                                                                         variable_values[param]))
                else:
                    logging.debug("Variable {} not found in evaluated variables in file {}".format(param, self.arm_file))
            except TypeError:
                logging.debug(f"Failed to evaluate param in {self.arm_file}", exc_info=True)

    @staticmethod
    def extract_arm_resource_id(arm_resource: dict[str, Any]) -> str | None:
        # if arm_resource_name == '__startline__' or arm_resource_name == '__endline__':
        #    return
        if 'type' not in arm_resource:
            # This is not an ARM resource, skip
            return None
        if 'name' not in arm_resource:
            # This is not an ARM resource, skip
            return None
        return f"{arm_resource['type']}.{arm_resource['name']}"

    @staticmethod
    def extract_arm_resource_name(arm_resource: dict[str, Any]) -> str | None:
        # if arm_resource_name == '__startline__' or arm_resource_name == '__endline__':
        #    return
        if 'name' not in arm_resource:
            # This is not an ARM resource, skip
            return None
        return f"{arm_resource['name']}"

    def extract_arm_resource_code_lines(
        self, arm_resource: dict[str, Any]
    ) -> tuple[list[int], list[tuple[int, str]]] | tuple[None, None]:
        find_lines_result_list = list(self.find_lines(arm_resource, START_LINE))
        if len(find_lines_result_list) >= 1:
            start_line = min(find_lines_result_list)
            end_line = max(list(self.find_lines(arm_resource, END_LINE)))

            entity_lines_range = [start_line, end_line]

            entity_code_lines = self.arm_template_lines[start_line - 1: end_line]
            return entity_lines_range, entity_code_lines
        return None, None

    @staticmethod
    def find_lines(node: dict[str, Any] | list[dict[str, Any]], kv: str) -> Generator[Any, None, None]:
        if isinstance(node, list):
            for i in node:
                for x in ContextParser.find_lines(i, kv):
                    yield x
        elif isinstance(node, dict):
            if kv in node:
                yield node[kv]
            for j in node.values():
                for x in ContextParser.find_lines(j, kv):
                    yield x

    @staticmethod
    def collect_skip_comments(resource: dict[str, Any]) -> list[_SkippedCheck]:
        skipped_checks = []
        bc_id_mapping = metadata_integration.bc_to_ckv_id_mapping
        if "metadata" in resource:
            if "checkov" in resource["metadata"]:
                for item in force_list(resource["metadata"]["checkov"]):
                    skip_search = re.search(COMMENT_REGEX, str(item))
                    if skip_search:
                        skipped_check: "_SkippedCheck" = {
                            'id': skip_search.group(1),
                            'suppress_comment': skip_search.group(2)[1:] if skip_search.group(
                                2) else "No comment provided"
                        }
                        if bc_id_mapping and skipped_check["id"] in bc_id_mapping:
                            skipped_check["bc_id"] = skipped_check["id"]
                            skipped_check["id"] = bc_id_mapping[skipped_check["id"]]
                        elif metadata_integration.check_metadata:
                            skipped_check["bc_id"] = metadata_integration.get_bc_id(skipped_check["id"])

                        skipped_checks.append(skipped_check)

        return skipped_checks

    @staticmethod
    def search_deep_keys(search_text: str, arm_dict: dict[str, Any], path: list[str | int]) -> list[list[Any]]:
        """Search deep for keys and get their values"""
        keys = []
        if isinstance(arm_dict, dict):
            for key in arm_dict:
                pathprop = path[:]
                pathprop.append(key)
                if key == search_text:
                    pathprop.append(arm_dict[key])
                    keys.append(pathprop)
                    # pop the last element off for nesting of found elements for
                    # dict and list checks
                    pathprop = pathprop[:-1]
                if isinstance(arm_dict[key], dict):
                    keys.extend(ContextParser.search_deep_keys(search_text, arm_dict[key], pathprop))
                elif isinstance(arm_dict[key], list):
                    for index, item in enumerate(arm_dict[key]):
                        pathproparr = pathprop[:]
                        pathproparr.append(index)
                        keys.extend(ContextParser.search_deep_keys(search_text, item, pathproparr))
        elif isinstance(arm_dict, list):
            for index, item in enumerate(arm_dict):
                pathprop = path[:]
                pathprop.append(index)
                keys.extend(ContextParser.search_deep_keys(search_text, item, pathprop))

        return keys

    @staticmethod
    def search_deep_values(search_text: str, arm_dict: dict[str, Any], path: list[str | int]) -> list[list[str | int]]:
        """Search deep for keys with values matching search text"""
        keys: "list[list[str | int]]" = []
        if isinstance(arm_dict, dict):
            for key in arm_dict:
                pathprop = path[:]
                pathprop.append(key)

                if search_text in str(arm_dict[key]):
                    pathprop.append(arm_dict[key])
                    keys.append(pathprop)
                    # pop the last element off for nesting of found elements for
                    # dict and list checks
                    pathprop = pathprop[:-1]
                if isinstance(arm_dict[key], dict):
                    keys.extend(ContextParser.search_deep_values(search_text, arm_dict[key], pathprop))
                elif isinstance(arm_dict[key], list):
                    for index, item in enumerate(arm_dict[key]):
                        pathproparr = pathprop[:]
                        pathproparr.append(index)
                        keys.extend(ContextParser.search_deep_values(search_text, item, pathproparr))
        elif isinstance(arm_dict, list):
            for index, item in enumerate(arm_dict):
                pathprop = path[:]
                pathprop.append(index)
                keys.extend(ContextParser.search_deep_values(search_text, item, pathprop))

        for inner_keys in keys:
            for i in inner_keys:
                if isinstance(i, list) or isinstance(i, dict):
                    keys.remove(inner_keys)

            # Remove parameter
            if search_text in inner_keys[-1]:  # type:ignore[operator]  # this will be a str
                inner_keys.pop()

        return keys

    def _set_in_dict(self, data_dict: dict[str, Any], map_list: list[str | int], value: Any) -> None:
        self._get_from_dict(data_dict, map_list[:-1])[map_list[-1]] = value  # type:ignore[index]  # this will be a str

    @staticmethod
    def _get_from_dict(data_dict: dict[str, Any], map_list: list[str | int]) -> dict[str, Any]:
        return reduce(operator.getitem, map_list, data_dict)  # type:ignore[arg-type]  # this works, because of a deeper dict access



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AuroraEncryption(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure all data stored in Aurora is securely encrypted at rest"
        id = "CKV_AWS_96"
        supported_resources = ("AWS::RDS::DBCluster",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, dict[str, Any]]) -> CheckResult:
        # If you specify the SnapshotIdentifier or SourceDBInstanceIdentifier property, don't specify this property.
        # The value is inherited from the snapshot or source DB instance.
        # https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html#cfn-rds-dbcluster-storageencrypted
        # Doc refers to 'SourceDBInstanceIdentifier' but that is not an available field. This is a doc error. 'SourceDBClusterIdentifier' is correct.
        properties = conf.get("Properties")
        if properties:
            if "SnapshotIdentifier" in properties.keys() or "SourceDBClusterIdentifier" in properties.keys():
                return CheckResult.UNKNOWN
        # No snapshot or source DB; Use base class implementation
        return super().scan_resource_conf(conf)

    def get_inspected_key(self) -> str:
        return "Properties/StorageEncrypted"


check = AuroraEncryption()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class LaunchTemplateMetadataHop(BaseResourceValueCheck):
    def __init__(self) -> None:
        """
        NIST.800-53.r5 CA-9(1), NIST.800-53.r5 CM-2, NIST.800-53.r5 CM-2(2)
        Auto Scaling group launch configuration should not have a metadata response hop limit greater than 1
        """
        name = "Ensure Launch template should not have a metadata response hop limit greater than 1"
        id = "CKV_AWS_341"
        supported_resources = ("aws_launch_configuration", "aws_launch_template")
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self) -> str:
        return "metadata_options/[0]/http_put_response_hop_limit"

    def get_expected_value(self):
        return 1


check = LaunchTemplateMetadataHop()



from __future__ import annotations

import os
from dataclasses import dataclass

CREATE_COMPLEX_VERTICES = 'CREATE_COMPLEX_VERTICES'
CREATE_EDGES = 'CREATE_EDGES'


@dataclass()
class K8sGraphFlags:
    create_complex_vertices: bool
    create_edges: bool

    def __init__(self, create_complex_vertices: bool = False, create_edges: bool = False) -> None:
        create_complex_vertices_env_var: bool = bool(os.environ.get(CREATE_COMPLEX_VERTICES, True))
        create_edges_env_var: bool = bool(os.environ.get(CREATE_EDGES, True))
        self.create_complex_vertices = create_complex_vertices or create_complex_vertices_env_var
        self.create_edges = create_edges or create_edges_env_var



# -*- coding: utf-8 -*-
"""
=================
stopit.signalstop
=================

Control the timeout of blocks or callables with a context manager or a
decorator. Based on the use of signal.SIGALRM
"""

from __future__ import annotations

import signal
from typing import TYPE_CHECKING

from .utils import TimeoutException, BaseTimeout, base_timeoutable

if TYPE_CHECKING:
    from types import FrameType


class SignalTimeout(BaseTimeout):
    """Context manager for limiting in the time the execution of a block
    using signal.SIGALRM Unix signal.

    See :class:`stopit.utils.BaseTimeout` for more information
    """

    def __init__(self, seconds: int, swallow_exc: bool = True) -> None:
        seconds = int(seconds)  # alarm delay for signal MUST be int
        super().__init__(seconds, swallow_exc)

    def handle_timeout(self, signum: int, frame: FrameType | None) -> None:
        self.state = self.TIMED_OUT
        raise TimeoutException(f"Block exceeded maximum timeout value ({self.seconds} seconds).")

    # Required overrides
    def setup_interrupt(self) -> None:
        signal.signal(signal.SIGALRM, self.handle_timeout)
        signal.alarm(self.seconds)

    def suppress_interrupt(self) -> None:
        signal.alarm(0)
        signal.signal(signal.SIGALRM, signal.SIG_DFL)


class signal_timeoutable(base_timeoutable):  # noqa: B903
    """A function or method decorator that raises a ``TimeoutException`` to
    decorated functions that should not last a certain amount of time.
    this one uses ``SignalTimeout`` context manager.

    See :class:`.utils.base_timoutable`` class for further comments.
    """
    def __init__(self) -> None:
        super().__init__()
        self.to_ctx_mgr = SignalTimeout



from __future__ import annotations

from typing import Dict, Set, Callable, Any

from checkov.common.sca.reachability.abstract_alias_mapping_strategy import AbstractAliasMappingStrategy
from checkov.common.sca.reachability.nodejs.utils import parse_webpack_file, parse_tsconfig_file, parse_babel_file, \
    parse_rollup_file, parse_package_json_file, parse_snowpack_file, parse_vite_file


class NodejsAliasMappingStrategy(AbstractAliasMappingStrategy):
    def get_language(self) -> str:
        return "nodejs"

    def get_file_name_to_parser_map(self) -> Dict[str, Callable[[str, Set[str]], Dict[str, Any]]]:
        return {
            "webpack.config.js": parse_webpack_file,
            "tsconfig.json": parse_tsconfig_file,
            ".babelrc": parse_babel_file,
            "babel.config.js": parse_babel_file,
            "rollup.config.js": parse_rollup_file,
            "package.json": parse_package_json_file,
            "snowpack.config.js": parse_snowpack_file,
            "vite.config.js": parse_vite_file
        }



from __future__ import annotations

from checkov.ansible.checks.base_ansible_task_value_check import BaseAnsibleTaskValueCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class UriValidateCerts(BaseAnsibleTaskValueCheck):
    def __init__(self) -> None:
        name = "Ensure that certificate validation isn't disabled with uri"
        id = "CKV_ANSIBLE_1"
        super().__init__(
            name=name,
            id=id,
            categories=(CheckCategories.GENERAL_SECURITY,),
            supported_modules=("ansible.builtin.uri", "uri"),
            missing_block_result=CheckResult.PASSED,
        )

    def get_inspected_key(self) -> str:
        return "validate_certs"


check = UriValidateCerts()
